@@ -1,28 +1,18 @@
-/*  $Id: cvtbatch.c,v 1.9 1999/11/19 09:13:48 rra Exp $
+/*  $Id: cvtbatch.c,v 1.21 2003/01/19 01:15:26 rra Exp $
 **
 **  Read file list on standard input and spew out batchfiles.
 */
+
 #include "config.h"
 #include "clibrary.h"
-#include <errno.h>
-#include <syslog.h>  
-#include <sys/stat.h>
 
+#include "inn/innconf.h"
+#include "inn/messages.h"
+#include "inn/qio.h"
+#include "inn/wire.h"
 #include "libinn.h"
-#include "macros.h"
 #include "paths.h"
-#include "qio.h"
-
-
-/*
-**  Print a usage message and exit.
-*/
-STATIC NORETURN
-Usage()
-{
-    (void)fprintf(stderr, "convertbatch usage_error.\n");
-    exit(1);
-}
+#include "storage.h"
 
 
 int
@@ -30,25 +20,27 @@ main(int ac, char *av[]) {
     int		i;
     QIOSTATE	*qp;
     char	*line;
-    char	*text;
+    const char	*text;
     char	*format;
     char	*p, *q;
-    BOOL	Dirty;
+    const char	*r;
+    bool	Dirty;
     TOKEN	token;
     ARTHANDLE	*art;
     int		len;
 
-    /* First thing, set up logging and our identity. */
-    openlog("cvtbatch", L_OPENLOG_FLAGS | LOG_PID, LOG_INN_PROG);           
+    /* First thing, set up our identity. */
+    message_program_name = "cvtbatch";
+    if (!innconf_read(NULL))
+        exit(1);
 
-    if (ReadInnConf() < 0) exit(1);
     /* Parse JCL. */
-    format = "nm";
+    format = xstrdup("nm");
     while ((i = getopt(ac, av, "w:")) != EOF)
 	switch (i) {
 	default:
-	    Usage();
-	    /* NOTREACHED */
+            die("usage error");
+            break;
 	case 'w':
 	    for (p = format = optarg; *p; p++) {
 		switch (*p) {
@@ -58,23 +50,16 @@ main(int ac, char *av[]) {
 		case FEED_NAME:
 		    continue;
 		}
-		(void)fprintf(stderr, "Ignoring \"%c\" in -w flag.\n", *p);
+                warn("ignoring %c in -w flag", *p);
 	    }
 	}
     ac -= optind;
     av += optind;
     if (ac)
-	Usage();
+	die("usage error");
 
-    if (chdir(innconf->patharticles) < 0) {
-	(void)fprintf(stderr, "batchconvert cant chdir %s, %s\n",
-		innconf->patharticles, strerror(errno));
-	exit(1);
-    }
-    if (!SMinit()) {
-	(void)fprintf(stderr, "cvtbatch: Could not initialize the storage manager: %s", SMerrorstr);
-	exit(1);
-    }
+    if (!SMinit())
+        die("cannot initialize storage manager: %s", SMerrorstr);
 
     /* Loop over all input. */
     qp = QIOfdopen((int)fileno(stdin));
@@ -90,51 +75,52 @@ main(int ac, char *av[]) {
 	token = TextToToken(line);
 	if ((art = SMretrieve(token, RETR_HEAD)) == NULL)
 	    continue;
-	if ((text = (char *)HeaderFindMem(art->data, art->len, "Message-ID", 10)) == NULL) {
+        text = wire_findheader(art->data, art->len, "Message-ID");
+	if (text == NULL) {
 	    SMfreearticle(art);
 	    continue;
 	}
 	len = art->len;
-	for (p = text; p < art->data + art->len; p++) {
-	    if (*p == '\r' || *p == '\n')
+	for (r = text; r < art->data + art->len; r++) {
+	    if (*r == '\r' || *r == '\n')
 		break;
 	}
-	if (p == art->data + art->len) {
+	if (r == art->data + art->len) {
 	    SMfreearticle(art);
 	    continue;
 	}
-	q = NEW(char, p - text + 1);
-	memcpy(q, text, p - text);
+	q = xmalloc(r - text + 1);
+	memcpy(q, text, r - text);
 	SMfreearticle(art);
-	q[p - text] = '\0';
+	q[r - text] = '\0';
 
 	/* Write the desired info. */
-	for (Dirty = FALSE, p = format; *p; p++) {
+	for (Dirty = false, p = format; *p; p++) {
 	    switch (*p) {
 	    default:
 		continue;
 	    case FEED_BYTESIZE:
 		if (Dirty)
-		    (void)putchar(' ');
-		(void)printf("%d", len);
+		    putchar(' ');
+		printf("%d", len);
 		break;
 	    case FEED_FULLNAME:
 	    case FEED_NAME:
 		if (Dirty)
-		    (void)putchar(' ');
-		(void)printf("%s", line);
+		    putchar(' ');
+		printf("%s", line);
 		break;
 	    case FEED_MESSAGEID:
 		if (Dirty)
-		    (void)putchar(' ');
-		(void)printf("%s", q);
+		    putchar(' ');
+		printf("%s", q);
 		break;
 	    }
-	    Dirty = TRUE;
+	    Dirty = true;
 	}
-	DISPOSE(q);
+	free(q);
 	if (Dirty)
-	    (void)putchar('\n');
+	    putchar('\n');
     }
 
     exit(0);
