@@ -1,4 +1,4 @@
-/*  $Id: hisv6.c 10305 2018-12-02 14:21:56Z iulius $
+/*  $Id: hisv6.c,v 1.11 2003/04/23 12:33:32 alexk Exp $
 **
 **  History v6 implementation against the history API.
 **
@@ -45,13 +45,11 @@
 #include "hisinterface.h"
 #include "hisv6.h"
 #include "hisv6-private.h"
-#include "inn/dbz.h"
-#include "inn/fdflag.h"
-#include "inn/innconf.h"
+#include "dbz.h"
 #include "inn/timer.h"
 #include "inn/qio.h"
 #include "inn/sequence.h"
-#include "inn/inndcomm.h"
+#include "inndcomm.h"
 
 /*
 **  because we can only have one open dbz per process, we keep a
@@ -79,9 +77,9 @@ hisv6_errloc(char *s, size_t line, off_t offset)
 {
     if (offset != -1) {
 	/* really we want an autoconf test for %ll/%L/%I64, sigh */
-        snprintf(s, HISV6_MAX_LOCATION, "@%lu", (unsigned long)offset);
+	sprintf(s, "@%.0f", (double)offset);
     } else {
-        snprintf(s, HISV6_MAX_LOCATION, ":%lu", (unsigned long)line);
+	sprintf(s, ":%lu", (unsigned long)line);
     }
 }
 
@@ -98,8 +96,7 @@ hisv6_splitline(const char *line, const char **error, HASH *hash,
 		 time_t *arrived, time_t *posted, time_t *expires,
 		 TOKEN *token)
 {
-    const char *p = line;
-    char *end;
+    char *p = (char *)line;
     unsigned long l;
     int r = 0;
 
@@ -124,8 +121,7 @@ hisv6_splitline(const char *line, const char **error, HASH *hash,
     }
 
     /* parse the arrived field */
-    l = strtoul(p + 1, &end, 10);
-    p = end;
+    l = strtoul(p + 1, &p, 10);
     if (l == ULONG_MAX) {
 	*error = "arrived timestamp out of range";
 	return -1;
@@ -147,8 +143,7 @@ hisv6_splitline(const char *line, const char **error, HASH *hash,
 	    if (expires)
 		*expires = 0;
 	} else {
-	    l = strtoul(p, &end, 10);
-            p = end;
+	    l = strtoul(p, &p, 10);
 	    if (l == ULONG_MAX) {
 		*error = "expires timestamp out of range";
 		return -1;
@@ -164,8 +159,7 @@ hisv6_splitline(const char *line, const char **error, HASH *hash,
 		*posted = 0;
 	} else {
 	    ++p;
-	    l = strtoul(p, &end, 10);
-            p = end;
+	    l = strtoul(p, &p, 10);
 	    if (l == ULONG_MAX) {
 		*error = "posted timestamp out of range";
 		return -1;
@@ -265,7 +259,6 @@ hisv6_closefiles(struct hisv6 *h)
 	    r = false;
 	}
 	h->writefp = NULL;
-        h->offset = 0;
     }
 
     h->nextcheck = 0;
@@ -293,7 +286,7 @@ hisv6_reopen(struct hisv6 *h)
 	if (h->flags & HIS_CREAT)
 	    mode = "w";
 	else
-	    mode = "r+";
+	    mode = "a";
 	if ((h->writefp = Fopen(h->histpath, mode, INND_HISTORY)) == NULL) {
 	    hisv6_seterror(h, concat("can't fopen history ",
 				      h->histpath, " ",
@@ -301,6 +294,12 @@ hisv6_reopen(struct hisv6 *h)
 	    hisv6_closefiles(h);
 	    goto fail;
 	}
+
+	/* fseeko to the end of file because the result of ftello() is
+	   undefined for files freopen()-ed in append mode according
+	   to POSIX 1003.1.  ftello() is used later on to determine a
+	   new article's offset in the history file. Fopen() uses
+	   freopen() internally. */
 	if (fseeko(h->writefp, 0, SEEK_END) == -1) {
 	    hisv6_seterror(h, concat("can't fseek to end of ",
 				      h->histpath, " ",
@@ -308,14 +307,7 @@ hisv6_reopen(struct hisv6 *h)
 	    hisv6_closefiles(h);
 	    goto fail;
 	}
-        h->offset = ftello(h->writefp);
-	if (h->offset == -1) {
-	    hisv6_seterror(h, concat("can't ftello ", h->histpath, " ",
-				      strerror(errno), NULL));
-	    hisv6_closefiles(h);
-	    goto fail;
-	}
-	fdflag_close_exec(fileno(h->writefp), true);
+	close_on_exec(fileno(h->writefp), true);
     }
 
     /* Open the history file for reading. */
@@ -325,7 +317,7 @@ hisv6_reopen(struct hisv6 *h)
 	hisv6_closefiles(h);
 	goto fail;
     }
-    fdflag_close_exec(h->readfd, true);
+    close_on_exec(h->readfd, true);
     
     /* if there's no current dbz owner, claim it here */
     if (hisv6_dbzowner == NULL) {
@@ -356,17 +348,6 @@ hisv6_reopen(struct hisv6 *h)
 	    /*opt.pag_incore = INCORE_NO;*/
 	    opt.pag_incore = (h->flags & HIS_MMAP) ? INCORE_MMAP : INCORE_NO;
 	    opt.exists_incore = (h->flags & HIS_MMAP) ? INCORE_MMAP : INCORE_NO;
-
-# if defined(MMAP_NEEDS_MSYNC) && INND_DBZINCORE == 1
-	    /* Systems that have MMAP_NEEDS_MSYNC defined will have their
-	       on-disk copies out of sync with the mmap'ed copies most of
-	       the time.  So if innd is using INCORE_MMAP, then we force
-	       everything else to use it, too (unless we're on NFS) */
-	    if(!innconf->nfsreader) {
-		opt.pag_incore = INCORE_MMAP;
-		opt.exists_incore = INCORE_MMAP;
-	    }
-# endif
 #endif
 	}
 	dbzsetoptions(opt);
@@ -469,7 +450,6 @@ hisv6_new(const char *path, int flags, struct history *history)
     h->histpath = path ? xstrdup(path) : NULL;
     h->flags = flags;
     h->writefp = NULL;
-    h->offset = 0;
     h->history = history;
     h->readfd = -1;
     h->nextcheck = 0;
@@ -723,11 +703,10 @@ hisv6_check(void *history, const char *key)
 
 
 /*
-**  Format a history line.  s should hold at least HISV6_MAXLINE + 1
-**  characters (to allow for the nul).  Returns the length of the data
-**  written, 0 if there was some error or if the data was too long to write.
+**  format a history line; `s' should be of at least HISV6_MAXLINE+1
+**  characters (to allow for the \0).
 */
-static int
+static bool
 hisv6_formatline(char *s, const HASH *hash, time_t arrived,
 		  time_t posted, time_t expires, const TOKEN *token)
 {
@@ -735,20 +714,9 @@ hisv6_formatline(char *s, const HASH *hash, time_t arrived,
     const char *hashtext = HashToText(*hash);
 
     if (token == NULL) {
-        /* Only a line to remember an article.  We keep its arrival
-         * and posting time. */
-        if (posted <= 0) {
-            i = snprintf(s, HISV6_MAXLINE, "[%s]%c%lu%c%c\n",
-                         hashtext, HISV6_FIELDSEP,
-                         (unsigned long)arrived, HISV6_SUBFIELDSEP,
-                         HISV6_NOEXP);
-        } else {
-            i = snprintf(s, HISV6_MAXLINE, "[%s]%c%lu%c%c%c%lu\n",
-                         hashtext, HISV6_FIELDSEP,
-                         (unsigned long)arrived, HISV6_SUBFIELDSEP,
-                         HISV6_NOEXP, HISV6_SUBFIELDSEP,
-                         (unsigned long)posted);
-        }
+	i = snprintf(s, HISV6_MAXLINE, "[%s]%c%lu%c%c\n",
+		     hashtext, HISV6_FIELDSEP,
+		     (unsigned long)arrived, HISV6_SUBFIELDSEP, HISV6_NOEXP);
     } else {
 	const char *texttok;
 
@@ -770,8 +738,8 @@ hisv6_formatline(char *s, const HASH *hash, time_t arrived,
 	}
     }
     if (i < 0 || i >= HISV6_MAXLINE)
-	return 0;
-    return i;
+	return false;
+    return true;
 }
 
 
@@ -825,8 +793,9 @@ static bool
 hisv6_writeline(struct hisv6 *h, const HASH *hash, time_t arrived,
 		time_t posted, time_t expires, const TOKEN *token)
 {
+    int i;
     bool r;
-    size_t i, length;
+    off_t offset;
     char hisline[HISV6_MAXLINE + 1];
     char location[HISV6_MAX_LOCATION];
 
@@ -842,31 +811,26 @@ hisv6_writeline(struct hisv6 *h, const HASH *hash, time_t arrived,
 	return false;
     }
 
-    length = hisv6_formatline(hisline, hash, arrived, posted, expires, token);
-    if (length == 0) {
+    if (hisv6_formatline(hisline, hash, arrived, posted, expires,
+			  token) != true) {
 	hisv6_seterror(h, concat("error formatting history line ",
 				  h->histpath, NULL));
 	return false;
     }	
 
-    i = fwrite(hisline, 1, length, h->writefp);
-
-    /* If the write failed, the history line is now an orphan.  Attempt to
-       rewind the write pointer to our offset to avoid leaving behind a
-       partial write and desyncing the offset from our file position. */
-    if (i < length ||
-        (!(h->flags & HIS_INCORE) && fflush(h->writefp) == EOF)) {
-	hisv6_errloc(location, (size_t)-1, h->offset);
+    offset = ftello(h->writefp);
+    i = fputs(hisline, h->writefp);
+    if (i == EOF ||
+	(!(h->flags & HIS_INCORE) && fflush(h->writefp) == EOF)) {
+	hisv6_errloc(location, (size_t)-1, offset);
+	/* The history line is now an orphan... */
 	hisv6_seterror(h, concat("can't write history ", h->histpath,
 				  location, " ", strerror(errno), NULL));
-        if (fseeko(h->writefp, h->offset, SEEK_SET) == -1)
-            h->offset += i;
 	r = false;
 	goto fail;
     }
 
-    r = hisv6_writedbz(h, hash, h->offset);
-    h->offset += length;     /* increment regardless of error from writedbz */
+    r = hisv6_writedbz(h, hash, offset);
  fail:
     return r;
 }
@@ -893,11 +857,10 @@ hisv6_write(void *history, const char *key, time_t arrived,
 
 
 /*
-**  Remember a history entry, key, with arrival time, and also
-**  posting time if known.
+**  remember a history entry, key, with arrival time arrived.
 */
 bool
-hisv6_remember(void *history, const char *key, time_t arrived, time_t posted)
+hisv6_remember(void *history, const char *key, time_t arrived)
 {
     struct hisv6 *h = history;
     HASH hash;
@@ -905,7 +868,7 @@ hisv6_remember(void *history, const char *key, time_t arrived, time_t posted)
 
     his_logger("HISwrite begin", S_HISwrite);
     hash = HashMessageID(key);
-    r = hisv6_writeline(h, &hash, arrived, posted, 0, NULL);
+    r = hisv6_writeline(h, &hash, arrived, 0, 0, NULL);
     his_logger("HISwrite end", S_HISwrite);
     return r;
 }
@@ -940,7 +903,7 @@ hisv6_replace(void *history, const char *key, time_t arrived,
 	char new[HISV6_MAXLINE + 1];
 
 	if (hisv6_formatline(new, &hash, arrived, posted, expires,
-                             token) == 0) {
+			      token) != true) {
  	    hisv6_seterror(h, concat("error formatting history line ",
 				      h->histpath, NULL));
 	    r = false;
@@ -949,9 +912,7 @@ hisv6_replace(void *history, const char *key, time_t arrived,
 
 	    oldlen = strlen(old);
 	    newlen = strlen(new);
-	    if (new[newlen - 1] == '\n')
-                newlen--;
-	    if (newlen > oldlen) {
+	    if (new > old) {
 		hisv6_seterror(h, concat("new history line too long ",
 					  h->histpath, NULL));
 		r = false;
@@ -959,12 +920,12 @@ hisv6_replace(void *history, const char *key, time_t arrived,
 		ssize_t n;
 
 		/* space fill any excess in the tail of new */
-		memset(new + newlen, ' ', oldlen - newlen);
+		memset(new + oldlen, ' ', oldlen - newlen);
 
 		do {
 		    n = pwrite(fileno(h->writefp), new, oldlen, offset);
 		} while (n == -1 && errno == EINTR);
-		if ((size_t) n != oldlen) {
+		if (n != oldlen) {
 		    char location[HISV6_MAX_LOCATION];
 
 		    hisv6_errloc(location, (size_t)-1, offset);
@@ -994,7 +955,7 @@ hisv6_traverse(struct hisv6 *h, struct hisv6_walkstate *cookie,
 				time_t, time_t, time_t,
 				const TOKEN *))
 {
-    bool r = false;
+    bool r;
     QIOSTATE *qp;
     void *p;
     size_t line;
@@ -1134,7 +1095,7 @@ hisv6_expirecb(struct hisv6 *h, void *cookie, const HASH *hash,
 
     /* check if we've seen this message id already */
     if (hiscookie->new && dbzexists(*hash)) {
-	/* continue after duplicates, it's serious, but not fatal */
+	/* continue after duplicates, it serious, but not fatal */
 	hisv6_seterror(h, concat("duplicate message-id [",
 				 HashToText(*hash), "] in history ",
 				 hiscookie->new->histpath, NULL));
@@ -1153,24 +1114,18 @@ hisv6_expirecb(struct hisv6 *h, void *cookie, const HASH *hash,
 	    keep = (*hiscookie->cb.expire)(hiscookie->cookie,
 					   arrived, posted, expires,
 					   t);
-	    /* If the callback returns true, we should keep the
-	     * token for the time being, else we just remember
-	     * it. */
+	    /* if the callback returns true, we should keep the
+	       token for the time being, else we just remember
+	       it */
 	    if (keep == false) {
 		t = NULL;
-		expires = 0;
+		posted = expires = 0;
 	    }
 	} else {
 	    t = NULL;
 	}
-        /* When t is NULL (no token), the message-ID is removed from
-         * history when the posting time of the article is older than
-         * threshold, as set by the /remember/ line in expire.ctl.
-         * We keep the check for the arrival time because some entries
-         * might not have one. */
 	if (hiscookie->new &&
-	    (t != NULL || posted >= hiscookie->threshold
-             || (posted <= 0 && arrived >= hiscookie->threshold))) {
+	    (t != NULL || arrived >= hiscookie->threshold)) {
 	    r = hisv6_writeline(hiscookie->new, hash,
 				 arrived, posted, expires, t);
 	}
@@ -1260,7 +1215,7 @@ hisv6_expire(void *history, const char *path, const char *reason,
 	     bool (*exists)(void *, time_t, time_t, time_t, TOKEN *))
 {
     struct hisv6 *h = history, *hnew = NULL;
-    char *nhistory = NULL;
+    const char *nhistory = NULL;
     dbzoptions opt;
     bool r;
     struct hisv6_walkstate hiscookie;
@@ -1288,8 +1243,7 @@ hisv6_expire(void *history, const char *path, const char *reason,
 	    nhistory = concat(h->histpath, ".n", NULL);
 	}
 
-	hnew = hisv6_new(nhistory, HIS_CREAT | HIS_RDWR | HIS_INCORE,
-                         h->history);
+	hnew = hisv6_new(nhistory, HIS_RDWR | HIS_INCORE, h->history);
 	if (!hisv6_reopen(hnew)) {
 	    hisv6_dispose(hnew);
 	    hnew = NULL;
@@ -1383,7 +1337,7 @@ hisv6_expire(void *history, const char *path, const char *reason,
     if (hnew && !hisv6_dispose(hnew))
 	r = false;
     if (nhistory && nhistory != path)
-	free(nhistory);
+	free((char *)nhistory);
     if (r == false && hiscookie.paused)
 	ICCgo(reason);
     return r;
