@@ -1,19 +1,35 @@
-/*  $Id: chan.c,v 1.44.2.2 2003/02/02 20:46:54 rra Exp $
+/*  $Id: chan.c,v 1.71 2003/02/10 01:24:17 rra Exp $
 **
 **  I/O channel (and buffer) processing.
 */
+
 #include "config.h"
 #include "clibrary.h"
-#include <netinet/in.h>
-#include <arpa/inet.h>
 
 /* Needed on AIX 4.1 to get fd_set and friends. */
 #ifdef HAVE_SYS_SELECT_H
 # include <sys/select.h>
 #endif
 
+#include "inn/innconf.h"
 #include "innd.h"
 
+/* These errno values don't exist on all systems, but may be returned as an
+   (ignorable) error to setting the accept socket nonblocking.  Define them
+   to 0 if they don't exist so that we can unconditionally compare errno to
+   them in the code. */
+#ifndef ENOTSOCK
+# define ENOTSOCK 0
+#endif
+#ifndef ENOTTY
+# define ENOTTY 0
+#endif
+
+static const char * const timer_name[] = {
+    "idle", "artclean", "artwrite", "artcncl", "sitesend", "overv",
+    "perl", "python", "nntpread", "artparse", "artlog", "datamove"
+};
+
 /* Minutes - basically, keep the connection open but idle */
 #define PAUSE_BEFORE_DROP               5
 
@@ -22,111 +38,44 @@
    readloop */
 #define COMP_THRESHOLD 10
 
-STATIC FDSET	RCHANmask;
-STATIC FDSET	SCHANmask;
-STATIC FDSET	WCHANmask;
-STATIC int	SCHANcount;
-STATIC int	CHANlastfd;
-STATIC int	CHANlastsleepfd;
-STATIC int	CHANccfd;
-STATIC int	CHANtablesize;
-STATIC CHANNEL	*CHANtable;
-STATIC CHANNEL	*CHANcc;
-STATIC CHANNEL	CHANnull = { CTfree, CSerror, -1 };
+static fd_set	RCHANmask;
+static fd_set	SCHANmask;
+static fd_set	WCHANmask;
+static int	SCHANcount;
+static int	CHANlastfd;
+static int	CHANlastsleepfd;
+static int	CHANccfd;
+static int	CHANtablesize;
+static CHANNEL	*CHANtable;
+static CHANNEL	*CHANcc;
+static CHANNEL	CHANnull = { CTfree, CSerror, -1 };
 
 #define PRIORITISE_REMCONN
 #ifdef PRIORITISE_REMCONN
-STATIC int	CHANrcfd;
-STATIC CHANNEL	*CHANrc;
+static int	*CHANrcfd;
+static CHANNEL	**CHANrc;
+static int	chanlimit;
 #endif /* PRIORITISE_REMCONN */
 
-/*
-**  Append data to a buffer.
-*/
-void BUFFappend(BUFFER *bp, const char *p, const int len) {
-    int i;
-    
-    if (len == 0)
-	return;
-    /* Note end of buffer, grow it if we need more room */
-    i = bp->Used + bp->Left;
-    if (i + len > bp->Size) {
-	/* Round size up to next 1K */
-	bp-> Size += (len + 0x3FF) & ~0x3FF;
-	RENEW(bp->Data, char, bp->Size);
-    }
-    bp->Left += len;
-    memcpy((POINTER)&bp->Data[i], (POINTER)p, len);
-}
-
-/*
-**  Set a buffer's contents, ignoring anything that might have
-**  been there.
-*/
-void BUFFset(BUFFER *bp, const char *p, const int length)
-{
-    if ((bp->Left = length) != 0) {
-	/* Need more space? */
-	if (bp->Size < length) {
-	    bp->Size = length;
-	    RENEW(bp->Data, char, bp->Size);
-	}
-	
-	/* Try to test for non-overlapping copies. */
-	memmove((POINTER)bp->Data, (POINTER)p, (SIZE_T)length);
-    }
-    bp->Used = 0;
-}
-
-/*
-**  Swap the contents of two buffers.
-*/
-void BUFFswap(BUFFER *b1, BUFFER *b2)
-{
-    BUFFER		b;
-
-    b = *b1;
-    *b1 = *b2;
-    *b2 = b;
-}
-
-/*
-**  Trim '\r' from buffer.
-*/
-void BUFFtrimcr(BUFFER *bp)
-{
-    char *p, *q;
-    int trimmed = 0;
-
-    for (p = q = bp->Data ; p < bp->Data + bp->Left ; p++) {
-	if (*p == '\r' && p+1 < bp->Data + bp->Left && p[1] == '\n') {
-	    trimmed++;
-	    continue;
-	}
-	*q++ = *p;
-    }
-    bp->Left -= trimmed;
-}
-
 /*
 ** Tear down our world
 */
 void
-CHANshutdown()
+CHANshutdown(void)
 {
   CHANNEL	        *cp;
   int			i;
 
   if (CHANtable) {
     for (i = CHANtablesize, cp = &CHANtable[0]; --i >= 0; cp++) {
-      if (cp->In.Data) {
-	DISPOSE(cp->In.Data);
+      if (cp->In.data) {
+	free(cp->In.data);
       }
-      if (cp->Out.Data) {
-	DISPOSE(cp->Out.Data);
+      if (cp->Out.data) {
+	free(cp->Out.data);
       }
     }
-    DISPOSE(CHANtable);
+    free(CHANtable);
     CHANtable = NULL;
   }
 }
@@ -134,7 +83,8 @@ CHANshutdown()
 /*
 **  Initialize all the I/O channels.
 */
-void CHANsetup(int i)
+void
+CHANsetup(int i)
 {
     CHANNEL	        *cp;
 
@@ -143,11 +93,9 @@ void CHANsetup(int i)
     FD_ZERO(&WCHANmask);
     CHANshutdown();
     CHANtablesize = i;
-    CHANtable = NEW(CHANNEL, CHANtablesize);
-    (void)memset((POINTER)CHANtable, 0,
-	    (SIZE_T)(CHANtablesize * sizeof *CHANtable));
+    CHANtable = xcalloc(CHANtablesize, sizeof(CHANNEL));
     CHANnull.NextLog = innconf->chaninacttime;
-    CHANnull.Address.s_addr = MyAddress.s_addr;
+    memset(&CHANnull.Address, 0, sizeof(CHANnull.Address));
     for (cp = CHANtable; --i >= 0; cp++)
 	*cp = CHANnull;
 }
@@ -156,39 +104,34 @@ void CHANsetup(int i)
 /*
 **  Create a channel from a descriptor.
 */
-CHANNEL *CHANcreate(int fd, CHANNELTYPE Type, CHANNELSTATE State,
-		    FUNCPTR Reader, FUNCPTR WriteDone)
+CHANNEL *
+CHANcreate(int fd, CHANNELTYPE Type, CHANNELSTATE State,
+           innd_callback_t Reader, innd_callback_t WriteDone)
 {
     CHANNEL	        *cp;
-    BUFFER		in;
-    BUFFER		out;
+    struct buffer	in  = { 0, 0, 0, NULL };
+    struct buffer	out = { 0, 0, 0, NULL };
 
     cp = &CHANtable[fd];
 
     /* Don't overwrite the buffers with CHANnull. */
     in = cp->In;
-    if (in.Size == 0) {
-	in.Size = START_BUFF_SIZE;
-	in.Data = NEW(char, in.Size);
-    }
-    in.Used = 0;
-    in.Left = in.Size;
+    buffer_resize(&in, START_BUFF_SIZE);
+    in.used = 0;
+    in.left = in.size;
     out = cp->Out;
-    if (out.Size == 0) {
-	out.Size = SMBUF;
-	out.Data = NEW(char, out.Size);
-    }
-    out.Used = 0;
-    out.Left = 0;
+    buffer_resize(&out, SMBUF);
+    buffer_set(&out, "", 0);
 
     /* Set up the channel's info. */
     *cp = CHANnull;
     cp->fd = fd;
     cp->Type = Type;
     cp->State = State;
-    cp->Streaming = FALSE;
-    cp->Skip = FALSE;
-    cp->NoResendId = FALSE;
+    cp->Streaming = false;
+    cp->Skip = false;
+    cp->NoResendId = false;
+    cp->privileged = false;
     cp->Ihave = cp->Ihave_Duplicate = cp->Ihave_Deferred = cp->Ihave_SendIt = cp->Ihave_Cybercan = 0;
     cp->Check = cp->Check_send = cp->Check_deferred = cp->Check_got = cp->Check_cybercan = 0;
     cp->Takethis = cp->Takethis_Ok = cp->Takethis_Err = 0;
@@ -201,33 +144,26 @@ CHANNEL *CHANcreate(int fd, CHANNELTYPE Type, CHANNELSTATE State,
     cp->In = in;
     cp->Out = out;
     cp->Tracing = Tracing;
-    cp->Sendid.Size = 0;
-    cp->Rest=0;
-    cp->SaveUsed=0;
-    cp->Lastch=0;
+    cp->Sendid.size = 0;
+    cp->Next=0;
     cp->MaxCnx=0;
     cp->ActiveCnx=0;
     cp->ArtBeg = 0;
     cp->ArtMax = 0;
+    cp->Start = 0;
     HashClear(&cp->CurrentMessageIDHash);
     memset(cp->PrecommitWIP, '\0', sizeof(cp->PrecommitWIP));
     cp->PrecommitiCachenext=0;
+    ARTprepare(cp);
 
-    /* Make the descriptor close-on-exec and non-blocking. */
-    CloseOnExec(fd, TRUE);
-#if !defined(_HPUX_SOURCE)
-    /* Stupid HPUX 11.0 has a broken listen/accept where setting the
-     * listensocket to nonblocking prevents you from successfully setting the
-     * socket returned by accept(2) back to blocking mode, no matter what,
-     * resulting in all kinds of funny behaviour, data loss, etc. etc.  */
-    if (SetNonBlocking(fd, TRUE) < 0
-#if	defined(ENOTSOCK)
-	    && errno != ENOTSOCK
-#endif	/* defined(ENOTSOCK) */
-#if	defined(ENOTTY)
-	    && errno != ENOTTY
-#endif	/* defined(ENOTTY) */
-    )
+    close_on_exec(fd, true);
+
+#ifndef _HPUX_SOURCE
+    /* Stupid HPUX 11.00 has a broken listen/accept where setting the listen
+       socket to nonblocking prevents you from successfully setting the
+       socket returned by accept(2) back to blocking mode, no matter what,
+       resulting in all kinds of funny behaviour, data loss, etc. etc.  */
+    if (nonblocking(fd, true) < 0 && errno != ENOTSOCK && errno != ENOTTY)
 	syslog(L_ERROR, "%s cant nonblock %d %m", LogName, fd);
 #endif
 
@@ -239,8 +175,38 @@ CHANNEL *CHANcreate(int fd, CHANNELTYPE Type, CHANNELSTATE State,
 #ifdef PRIORITISE_REMCONN
     /* Note remconn channel, for efficiency */
     if (Type == CTremconn) {
-	CHANrc = cp;
-	CHANrcfd = fd;
+	int j;
+	for (j = 0 ; j < chanlimit ; j++ ) {
+	    if (CHANrcfd[j] == -1) {
+		break;
+	    }
+	}
+	if (j < chanlimit) {
+	    CHANrc[j] = cp;
+	    CHANrcfd[j] = fd;
+	} else if (chanlimit == 0) {
+	    /* assuming two file descriptors(AF_INET and AF_INET6) */
+	    chanlimit = 2;
+	    CHANrc = xmalloc(chanlimit * sizeof(CHANNEL **));
+	    CHANrcfd = xmalloc(chanlimit * sizeof(int *));
+	    for (j = 0 ; j < chanlimit ; j++ ) {
+		CHANrc[j] = NULL;
+		CHANrcfd[j] = -1;
+	    }
+	    CHANrc[0] = cp;
+	    CHANrcfd[0] = fd;
+	} else {
+	    /* extend to double size */
+            CHANrc = xrealloc(CHANrc, chanlimit * 2 * sizeof(CHANNEL **));
+            CHANrcfd = xrealloc(CHANrcfd, chanlimit * 2 * sizeof(int *));
+	    for (j = chanlimit ; j < chanlimit * 2 ; j++ ) {
+		CHANrc[j] = NULL;
+		CHANrcfd[j] = -1;
+	    }
+	    CHANrc[chanlimit] = cp;
+	    CHANrcfd[chanlimit] = fd;
+	    chanlimit *= 2;
+	}
     }
 #endif /* PRIORITISE_REMCONN */
     return cp;
@@ -250,7 +216,8 @@ CHANNEL *CHANcreate(int fd, CHANNELTYPE Type, CHANNELSTATE State,
 /*
 **  Start tracing a channel.
 */
-void CHANtracing(CHANNEL *cp, BOOL Flag)
+void
+CHANtracing(CHANNEL *cp, bool Flag)
 {
     char		*p;
 
@@ -261,16 +228,17 @@ void CHANtracing(CHANNEL *cp, BOOL Flag)
 	syslog(L_NOTICE, "%s trace badwrites %d blockwrites %d badreads %d",
 	    p, cp->BadWrites, cp->BlockedWrites, cp->BadReads);
 	syslog(L_NOTICE, "%s trace address %s lastactive %ld nextlog %ld",
-	    p, inet_ntoa(cp->Address), cp->LastActive, cp->NextLog);
+	    p, sprint_sockaddr((struct sockaddr *)&cp->Address),
+	    cp->LastActive, cp->NextLog);
 	if (FD_ISSET(cp->fd, &SCHANmask))
-	    syslog(L_NOTICE, "%s trace sleeping %ld 0x%x",
-		p, (long)cp->Waketime, cp->Waker);
+	    syslog(L_NOTICE, "%s trace sleeping %ld 0x%p",
+		p, (long)cp->Waketime, (void *)cp->Waker);
 	if (FD_ISSET(cp->fd, &RCHANmask))
-	    syslog(L_NOTICE, "%s trace reading %ld %s",
-		p, cp->In.Used, MaxLength(cp->In.Data, cp->In.Data));
+	    syslog(L_NOTICE, "%s trace reading %d %s",
+		p, cp->In.used, MaxLength(cp->In.data, cp->In.data));
 	if (FD_ISSET(cp->fd, &WCHANmask))
-	    syslog(L_NOTICE, "%s trace writing %ld %s",
-		p, cp->Out.Left, MaxLength(cp->Out.Data, cp->Out.Data));
+	    syslog(L_NOTICE, "%s trace writing %d %s",
+		p, cp->Out.left, MaxLength(cp->Out.data, cp->Out.data));
     }
 }
 
@@ -278,9 +246,10 @@ void CHANtracing(CHANNEL *cp, BOOL Flag)
 /*
 **  Close a channel.
 */
-void CHANclose(CHANNEL *cp, char *name)
+void
+CHANclose(CHANNEL *cp, const char *name)
 {
-    char	*label, *tmplabel;
+    char	*label, *tmplabel, buff[SMBUF];
 
     if (cp->Type == CTfree)
 	syslog(L_ERROR, "%s internal closing free channel %d", name, cp->fd);
@@ -288,15 +257,61 @@ void CHANclose(CHANNEL *cp, char *name)
 	if (cp->Type == CTnntp) {
 	    WIPprecomfree(cp);
 	    NCclearwip(cp);
+            if (cp->State == CScancel)
+                syslog(L_NOTICE,
+               "%s closed seconds %ld cancels %ld",
+               name, (long)(Now.time - cp->Started),
+               cp->Received);
+            else {
+	    snprintf(buff, sizeof(buff),
+                     "accepted size %.0f duplicate size %.0f", cp->Size,
+                     cp->DuplicateSize);
 	    syslog(L_NOTICE,
-		"%s closed seconds %ld accepted %ld refused %ld rejected %ld duplicate %ld accepted size %.0f duplicate size %.0f",
+		"%s closed seconds %ld accepted %ld refused %ld rejected %ld duplicate %ld %s",
 		name, (long)(Now.time - cp->Started),
 		cp->Received, cp->Refused, cp->Rejected,
-		cp->Duplicate, cp->Size, cp->DuplicateSize);
+		cp->Duplicate, buff);
+	    }
+	    if (cp->Data.Newsgroups.Data != NULL) {
+		free(cp->Data.Newsgroups.Data);
+		cp->Data.Newsgroups.Data = NULL;
+	    }
+	    if (cp->Data.Newsgroups.List != NULL) {
+		free(cp->Data.Newsgroups.List);
+		cp->Data.Newsgroups.List = NULL;
+	    }
+	    if (cp->Data.Distribution.Data != NULL) {
+		free(cp->Data.Distribution.Data);
+		cp->Data.Distribution.Data = NULL;
+	    }
+	    if (cp->Data.Distribution.List != NULL) {
+		free(cp->Data.Distribution.List);
+		cp->Data.Distribution.List = NULL;
+	    }
+	    if (cp->Data.Path.Data != NULL) {
+		free(cp->Data.Path.Data);
+		cp->Data.Path.Data = NULL;
+	    }
+	    if (cp->Data.Path.List != NULL) {
+		free(cp->Data.Path.List);
+		cp->Data.Path.List = NULL;
+	    }
+	    if (cp->Data.Overview.size != 0) {
+		free(cp->Data.Overview.data);
+		cp->Data.Overview.data = NULL;
+		cp->Data.Overview.size = 0;
+                cp->Data.Overview.left = 0;
+                cp->Data.Overview.used = 0;
+	    }
+	    if (cp->Data.XrefBufLength != 0) {
+		free(cp->Data.Xref);
+		cp->Data.Xref = NULL;
+		cp->Data.XrefBufLength = 0;
+	    }
 	} else if (cp->Type == CTreject)
 	    syslog(L_NOTICE, "%s %ld", name, cp->Rejected);
-	else if (cp->Out.Left)
-	    syslog(L_NOTICE, "%s closed lost %ld", name, cp->Out.Left);
+	else if (cp->Out.left)
+	    syslog(L_NOTICE, "%s closed lost %d", name, cp->Out.left);
 	else
 	    syslog(L_NOTICE, "%s closed", name);
 	WCHANremove(cp);
@@ -304,7 +319,7 @@ void CHANclose(CHANNEL *cp, char *name)
 	SCHANremove(cp);
 	if (cp->Argument != NULL)
 	    /* Set to NULL below. */
-	    DISPOSE(cp->Argument);
+	    free(cp->Argument);
 	if (cp->fd >= 0 && close(cp->fd) < 0)
 	    syslog(L_ERROR, "%s cant close %s %m", LogName, name);
  
@@ -316,7 +331,7 @@ void CHANclose(CHANNEL *cp, char *name)
 	    if ((label = RClabelname(cp)) != NULL) {
 		for(tfd = 0; tfd <= CHANlastfd; tfd++) {
 		    tempchan = &CHANtable[tfd];
-		    if(tempchan->fd > 0 &&
+		    if(tempchan->fd > 0 && tempchan->Type == CTnntp &&
 			((tmplabel = RClabelname(tempchan)) != NULL) &&
 			strcmp(label, tmplabel) == 0 &&
 			tempchan->ActiveCnx == 0) {
@@ -337,19 +352,26 @@ void CHANclose(CHANNEL *cp, char *name)
     cp->ActiveCnx = 0;
 
     /* Free the buffers if they got big. */
-    if (cp->In.Size > BIG_BUFFER) {
-	cp->In.Size = 0;
-	DISPOSE(cp->In.Data);
-	cp->In.Data = NULL; /* avoid spurious free()s of already freed data in CHANshutdown */
+    if (cp->In.size > BIG_BUFFER) {
+	cp->In.size = 0;
+        cp->In.used = 0;
+        cp->In.left = 0;
+	free(cp->In.data);
+	cp->In.data = NULL;
     }
-    if (cp->Out.Size > BIG_BUFFER) {
-	cp->Out.Size = 0;
-	DISPOSE(cp->Out.Data);
-	cp->Out.Data = NULL; /* ditto */
+    if (cp->Out.size > BIG_BUFFER) {
+	cp->Out.size = 0;
+        cp->Out.used = 0;
+        cp->Out.left = 0;
+	free(cp->Out.data);
+	cp->Out.data = NULL;
     }
-    if (cp->Sendid.Size) {
-	cp->Sendid.Size = 0;
-	DISPOSE(cp->Sendid.Data);
+    if (cp->Sendid.size > 0) {
+	cp->Sendid.size = 0;
+	cp->Sendid.used = 0;
+	cp->Sendid.left = 0;
+	free(cp->Sendid.data);
+        cp->Sendid.data = NULL;
     }
 }
 
@@ -357,40 +379,42 @@ void CHANclose(CHANNEL *cp, char *name)
 /*
 **  Return a printable name for the channel.
 */
-char *CHANname(const CHANNEL *cp)
+char *
+CHANname(const CHANNEL *cp)
 {
     static char		buff[SMBUF];
     int	                i;
-    SITE	        *sp;
-    STRING		p;
-    PID_T		pid;
+    SITE *	        sp;
+    const char *	p;
+    pid_t		pid;
 
     switch (cp->Type) {
     default:
-	(void)sprintf(buff, "?%d(#%d@%d)?", cp->Type, cp->fd, cp - CHANtable);
+	snprintf(buff, sizeof(buff), "?%d(#%d@%d)?", cp->Type, cp->fd,
+                 cp - CHANtable);
 	break;
     case CTany:
-	(void)sprintf(buff, "any:%d", cp->fd);
+	snprintf(buff, sizeof(buff), "any:%d", cp->fd);
 	break;
     case CTfree:
-	(void)sprintf(buff, "free:%d", cp->fd);
+	snprintf(buff, sizeof(buff), "free:%d", cp->fd);
 	break;
     case CTremconn:
-	(void)sprintf(buff, "remconn:%d", cp->fd);
+	snprintf(buff, sizeof(buff), "remconn:%d", cp->fd);
 	break;
     case CTreject:
-	(void)sprintf(buff, "%s rejected", RChostname(cp));
+	snprintf(buff, sizeof(buff), "%s rejected", RChostname(cp));
 	break;
     case CTnntp:
-	(void)sprintf(buff, "%s:%d",
-		cp->Address.s_addr == 0 ? "localhost" : RChostname(cp),
-		cp->fd);
+	snprintf(buff, sizeof(buff), "%s:%d",
+                 cp->Address.ss_family == 0 ? "localhost" : RChostname(cp),
+                 cp->fd);
 	break;
     case CTlocalconn:
-	(void)sprintf(buff, "localconn:%d", cp->fd);
+	snprintf(buff, sizeof(buff), "localconn:%d", cp->fd);
 	break;
     case CTcontrol:
-	(void)sprintf(buff, "control:%d", cp->fd);
+	snprintf(buff, sizeof(buff), "control:%d", cp->fd);
 	break;
     case CTexploder:
     case CTfile:
@@ -404,13 +428,13 @@ char *CHANname(const CHANNEL *cp)
 		break;
 	    }
 	if (pid == 0)
-	    (void)sprintf(buff, "%s:%d:%s",
-		    MaxLength(p, p), cp->fd,
-		    cp->Type == CTfile ? "file" : "proc");
+	    snprintf(buff, sizeof(buff), "%s:%d:%s",
+                     MaxLength(p, p), cp->fd,
+                     cp->Type == CTfile ? "file" : "proc");
 	else
-	    (void)sprintf(buff, "%s:%d:%s:%ld",
-		    MaxLength(p, p), cp->fd,
-		    cp->Type == CTfile ? "file" : "proc", (long)pid);
+	    snprintf(buff, sizeof(buff), "%s:%d:%s:%ld",
+                     MaxLength(p, p), cp->fd,
+                     cp->Type == CTfile ? "file" : "proc", (long)pid);
 	break;
     }
     return buff;
@@ -420,7 +444,8 @@ char *CHANname(const CHANNEL *cp)
 /*
 **  Return the channel for a specified descriptor.
 */
-CHANNEL *CHANfromdescriptor(int fd)
+CHANNEL *
+CHANfromdescriptor(int fd)
 {
     if (fd <0 || fd > CHANtablesize)
 	return NULL;
@@ -431,7 +456,8 @@ CHANNEL *CHANfromdescriptor(int fd)
 /*
 **  Iterate over all channels of a specified type.
 */
-CHANNEL *CHANiter(int *ip, CHANNELTYPE Type)
+CHANNEL *
+CHANiter(int *ip, CHANNELTYPE Type)
 {
     CHANNEL	        *cp;
     int	                i;
@@ -454,21 +480,24 @@ CHANNEL *CHANiter(int *ip, CHANNELTYPE Type)
 /*
 **  Mark a channel as an active reader.
 */
-void RCHANadd(CHANNEL *cp)
+void
+RCHANadd(CHANNEL *cp)
 {
     FD_SET(cp->fd, &RCHANmask);
     if (cp->fd > CHANlastfd)
 	CHANlastfd = cp->fd;
 
-    /* Start reading at the beginning of the buffer. */
-    cp->In.Used = 0;
+    if (cp->Type != CTnntp)
+	/* Start reading at the beginning of the buffer. */
+	cp->In.used = 0;
 }
 
 
 /*
 **  Remove a channel from the set of readers.
 */
-void RCHANremove(CHANNEL *cp)
+void
+RCHANremove(CHANNEL *cp)
 {
     if (FD_ISSET(cp->fd, &RCHANmask)) {
 	FD_CLR(cp->fd, &RCHANmask);
@@ -487,7 +516,9 @@ void RCHANremove(CHANNEL *cp)
 **  Put a channel to sleep, call a function when it wakes.
 **  Note that the Argument must be NULL or allocated memory!
 */
-void SCHANadd(CHANNEL *cp, time_t Waketime, POINTER Event, FUNCPTR Waker, POINTER Argument)
+void
+SCHANadd(CHANNEL *cp, time_t Waketime, void *Event, innd_callback_t Waker,
+         void *Argument)
 {
     if (!FD_ISSET(cp->fd, &SCHANmask)) {
 	SCHANcount++;
@@ -498,7 +529,7 @@ void SCHANadd(CHANNEL *cp, time_t Waketime, POINTER Event, FUNCPTR Waker, POINTE
     cp->Waketime = Waketime;
     cp->Waker = Waker;
     if (cp->Argument != Argument) {
-	DISPOSE(cp->Argument);
+	free(cp->Argument);
 	cp->Argument = Argument;
     }
     cp->Event = Event;
@@ -508,7 +539,8 @@ void SCHANadd(CHANNEL *cp, time_t Waketime, POINTER Event, FUNCPTR Waker, POINTE
 /*
 **  Take a channel off the sleep list.
 */
-void SCHANremove(CHANNEL *cp)
+void
+SCHANremove(CHANNEL *cp)
 {
     if (FD_ISSET(cp->fd, &SCHANmask)) {
 	FD_CLR(cp->fd, &SCHANmask);
@@ -527,7 +559,8 @@ void SCHANremove(CHANNEL *cp)
 /*
 **  Is a channel on the sleep list?
 */
-BOOL CHANsleeping(CHANNEL *cp)
+bool
+CHANsleeping(CHANNEL *cp)
 {
     return FD_ISSET(cp->fd, &SCHANmask);
 }
@@ -536,7 +569,8 @@ BOOL CHANsleeping(CHANNEL *cp)
 /*
 **  Wake up channels waiting for a specific event.
 */
-void SCHANwakeup(POINTER *Event)
+void
+SCHANwakeup(void *Event)
 {
     CHANNEL	        *cp;
     int	                i;
@@ -548,12 +582,13 @@ void SCHANwakeup(POINTER *Event)
 
 
 /*
-**  Mark a channel as an active writer.  Don't reset the Out->Left field
+**  Mark a channel as an active writer.  Don't reset the Out->left field
 **  since we could have buffered I/O already in there.
 */
-void WCHANadd(CHANNEL *cp)
+void
+WCHANadd(CHANNEL *cp)
 {
-    if (cp->Out.Left > 0) {
+    if (cp->Out.left > 0) {
 	FD_SET(cp->fd, &WCHANmask);
 	if (cp->fd > CHANlastfd)
 	    CHANlastfd = cp->fd;
@@ -564,14 +599,15 @@ void WCHANadd(CHANNEL *cp)
 /*
 **  Remove a channel from the set of writers.
 */
-void WCHANremove(CHANNEL *cp)
+void
+WCHANremove(CHANNEL *cp)
 {
     if (FD_ISSET(cp->fd, &WCHANmask)) {
 	FD_CLR(cp->fd, &WCHANmask);
-	if (cp->Out.Left <= 0) {
+	if (cp->Out.left <= 0) {
 	    /* No data left -- reset used so we don't grow the buffer. */
-	    cp->Out.Used = 0;
-	    cp->Out.Left = 0;
+	    cp->Out.used = 0;
+	    cp->Out.left = 0;
 	}
 	if (cp->fd == CHANlastfd) {
 	    /* This was the highest descriptor, get a new highest. */
@@ -587,9 +623,10 @@ void WCHANremove(CHANNEL *cp)
 /*
 **  Set a channel to start off with the contents of an existing channel.
 */
-void WCHANsetfrombuffer(CHANNEL *cp, BUFFER *bp)
+void
+WCHANsetfrombuffer(CHANNEL *cp, struct buffer *bp)
 {
-    WCHANset(cp, &bp->Data[bp->Used], bp->Left);
+    WCHANset(cp, &bp->data[bp->used], bp->left);
 }
 
 
@@ -597,45 +634,79 @@ void WCHANsetfrombuffer(CHANNEL *cp, BUFFER *bp)
 /*
 **  Read in text data, return the amount we read.
 */
-int CHANreadtext(CHANNEL *cp)
+int
+CHANreadtext(CHANNEL *cp)
 {
-    int	                i;
-    BUFFER	        *bp;
+    int	                i, j;
+    struct buffer       *bp;
     char		*p;
     int			oerrno;
     int			maxbyte;
+    HDRCONTENT		*hc = cp->Data.HdrContent;
 
-    /* Grow buffer if we're getting close to current limit. */
+    /* Grow buffer if we're getting close to current limit.  FIXME: The In
+       buffer doesn't use the normal meanings of .used and .left.  */
     bp = &cp->In;
-    bp->Left = bp->Size - bp->Used;
-    if (bp->Left <= LOW_WATER) {
-	i = GROW_AMOUNT(bp->Size);
-	bp->Size += i;
-	bp->Left += i;
-	RENEW(bp->Data, char, bp->Size);
+    bp->left = bp->size - bp->used;
+    if (bp->left <= LOW_WATER) {
+	i = GROW_AMOUNT(bp->size);
+	bp->size += i;
+	bp->left += i;
+	p = bp->data;
+	TMRstart(TMR_DATAMOVE);
+        bp->data = xrealloc(bp->data, bp->size);
+
+	/* Adjust offets of realloc moved the location of the memory region.
+           FIXME: This is invalid C, although it will work on most (all?)
+           common systems.  The pointers need to be reduced to offets and then
+           turned back into relative pointers rather than adjusting the
+           pointers directly, since as soon as realloc is called, pointers
+           into the old space become invalid and may not be used further. */
+	if ((i = p - bp->data) != 0) {
+	    if (cp->State == CSgetheader || cp->State == CSgetbody ||
+		cp->State == CSeatarticle) {
+		/* adjust offset only in CSgetheader, CSgetbody or
+		   CSeatarticle */
+		if (cp->Data.BytesHeader != NULL)
+		  cp->Data.BytesHeader -= i;
+		for (j = 0 ; j < MAX_ARTHEADER ; j++, hc++) {
+		    if (hc->Value != NULL)
+			hc->Value -= i;
+		}
+	    }
+	}
+	TMRstop(TMR_DATAMOVE);
     }
 
-    /* Read in whatever is there, up to some reasonable limit. */
-    /*
-     * XXX We really want to limit the amount of time it takes to
-     * process the incoming data for this channel.  But there's
-     * no easy way of doing that, so we restrict the data size instead.
-     * If the data is part of a single large article, then reading
-     * and processing many kilobytes at a time costs very little.
-     * If the data is a long list of CHECK commands from a streaming
-     * feed, then every line of data will require a history lookup, and
-     * we probably don't want to do more than about 10 of those per
-     * channel on each cycle of the main select() loop (otherwise we
-     * might take too long before giving other channels a turn).  10
-     * lines of CHECK commands suggests a limit of about 1 kilobyte of
-     * data, or less.  BUFSIZ is often about 1 kilobyte, and is
-     * attractive for other reasons, so let's use that as our size limit.
-     */
-    /*
-     * Reduce the read size only if we are reading commands.
-     */
-    maxbyte = (cp->State != CSgetcmd || bp->Left < BUFSIZ) ? bp->Left : BUFSIZ;
-    i = read(cp->fd, &bp->Data[bp->Used], maxbyte-1);
+    /* Read in whatever is there, up to some reasonable limit.
+
+       We want to limit the amount of time devoted to processing the incoming
+       data for any given channel.  There's no easy way of doing that, though,
+       so we restrict the data size instead.
+
+       If the data is part of a single large article, then reading and
+       processing many kilobytes at a time costs very little.  If the data is
+       a long list of CHECK commands from a streaming feed, then every line of
+       data will require a history lookup, and we probably don't want to do
+       more than about 10 of those per channel on each cycle of the main
+       select() loop (otherwise we might take too long before giving other
+       channels a turn).  10 lines of CHECK commands suggests a limit of about
+       1KB of data, or less.  innconf->maxcmdreadsize (BUFSIZ by default) is
+       often about 1KB, and is attractive for other reasons, so let's use that
+       as our size limit.  If innconf->maxcmdreadsize is 0, there is no limit.
+
+       Reduce the read size only if we're reading commands.
+
+       FIXME: A better approach would be to limit the number of commands we
+       process for each channel. */
+    if (innconf->maxcmdreadsize <= 0 || cp->State != CSgetcmd
+        || bp->left < innconf->maxcmdreadsize)
+        maxbyte = bp->left;
+    else
+        maxbyte = innconf->maxcmdreadsize;
+    TMRstart(TMR_NNTPREAD);
+    i = read(cp->fd, &bp->data[bp->used], maxbyte);
+    TMRstop(TMR_NNTPREAD);
     if (i < 0) {
         /* Solaris (at least 2.4 through 2.6) will occasionally return
            EAGAIN in response to a read even if the file descriptor already
@@ -644,23 +715,22 @@ int CHANreadtext(CHANNEL *cp)
            back out to the main loop and go on to the next file descriptor,
            as if the descriptor never selected true.  This check will
            probably never trigger on platforms other than Solaris. */
-        if (errno == EAGAIN) {
+        if (errno == EAGAIN)
             return -2;
-        }
 	oerrno = errno;
 	p = CHANname(cp);
 	errno = oerrno;
-	syslog(L_ERROR, "%s cant read %m", p);
+	sysnotice("%s cant read", p);
 	return -1;
     }
     if (i == 0) {
 	p = CHANname(cp);
-	syslog(L_NOTICE, "%s readclose", p);
+	notice("%s readclose", p);
 	return 0;
     }
 
-    bp->Used += i;
-    bp->Left -= i;
+    bp->used += i;
+    bp->left -= i;
     return i;
 }
 
@@ -670,7 +740,7 @@ int CHANreadtext(CHANNEL *cp)
 **  happens we want to do the I/O in chunks.  We assume stdio's BUFSIZ is
 **  a good chunk value.
 */
-STATIC int
+static int
 CHANwrite(int fd, char *p, long length)
 {
     int	i;
@@ -678,14 +748,14 @@ CHANwrite(int fd, char *p, long length)
 
     do {
 	/* Try the standard case -- write it all. */
-	i = write(fd, (POINTER)p, (SIZE_T)length);
+	i = write(fd, p, length);
 	if (i > 0 || (i < 0 && errno != EMSGSIZE && errno != EINTR))
 	    return i;
     } while (i < 0 && errno == EINTR);
 
     /* Write it in pieces. */
     for (save = p, i = 0; length; p += i, length -= i) {
-	i = write(fd, (POINTER)p, (SIZE_T)(length > BUFSIZ ? BUFSIZ : length));
+	i = write(fd, p, (length > BUFSIZ ? BUFSIZ : length));
 	if (i <= 0)
 	    break;
     }
@@ -698,27 +768,28 @@ CHANwrite(int fd, char *p, long length)
 /*
 **  Try to flush out the buffer.  Use this only on file channels!
 */
-BOOL WCHANflush(CHANNEL *cp)
+bool
+WCHANflush(CHANNEL *cp)
 {
-    BUFFER	        *bp;
+    struct buffer       *bp;
     int	                i;
 
     /* Write it. */
-    for (bp = &cp->Out; bp->Left > 0; bp->Left -= i, bp->Used += i) {
-	i = CHANwrite(cp->fd, &bp->Data[bp->Used], bp->Left);
+    for (bp = &cp->Out; bp->left > 0; bp->left -= i, bp->used += i) {
+	i = CHANwrite(cp->fd, &bp->data[bp->used], bp->left);
 	if (i < 0) {
-	    syslog(L_ERROR, "%s cant flush count %ld %m",
-		CHANname(cp), bp->Left);
-	    return FALSE;
+	    syslog(L_ERROR, "%s cant flush count %d %m",
+		CHANname(cp), bp->left);
+	    return false;
 	}
 	if (i == 0) {
-	    syslog(L_ERROR, "%s cant flush count %ld",
-		CHANname(cp), bp->Left);
-	    return FALSE;
+	    syslog(L_ERROR, "%s cant flush count %d",
+		CHANname(cp), bp->left);
+	    return false;
 	}
     }
     WCHANremove(cp);
-    return TRUE;
+    return true;
 }
 
 
@@ -726,7 +797,7 @@ BOOL WCHANflush(CHANNEL *cp)
 /*
 **  Wakeup routine called after a write channel was put to sleep.
 */
-STATIC FUNCTYPE
+static void
 CHANwakeup(CHANNEL *cp)
 {
     syslog(L_NOTICE, "%s wakeup", CHANname(cp));
@@ -737,7 +808,7 @@ CHANwakeup(CHANNEL *cp)
 /*
 **  Attempting to write would block; stop output or give up.
 */
-STATIC void
+static void
 CHANwritesleep(CHANNEL *cp, char *p)
 {
     int			i;
@@ -758,8 +829,7 @@ CHANwritesleep(CHANNEL *cp, char *p)
 	}
     i *= innconf->blockbackoff;
     syslog(L_ERROR, "%s blocked sleeping %d", p, i);
-    SCHANadd(cp, (time_t)(Now.time + i), (POINTER)NULL,
-	CHANwakeup, (POINTER)NULL);
+    SCHANadd(cp, Now.time + i, NULL, CHANwakeup, NULL);
 }
 
 
@@ -768,10 +838,10 @@ CHANwritesleep(CHANNEL *cp, char *p)
 **  We got an unknown error in select.  Find out the culprit.
 **  Not really ready for production use yet, and it's expensive, too.
 */
-STATIC void
+static void
 CHANdiagnose(void)
 {
-    FDSET		Test;
+    fd_set		Test;
     int			i;
     struct timeval	t;
 
@@ -781,7 +851,7 @@ CHANdiagnose(void)
 	    FD_SET(i, &Test);
 	    t.tv_sec = 0;
 	    t.tv_usec = 0;
-	    if (select(i + 1, &Test, (FDSET *)NULL, (FDSET *)NULL, &t) < 0
+	    if (select(i + 1, &Test, NULL, NULL, &t) < 0
 	      && errno != EINTR) {
 		syslog(L_ERROR, "%s Bad Read File %d", LogName, i);
 		FD_CLR(i, &RCHANmask);
@@ -794,7 +864,7 @@ CHANdiagnose(void)
 	    FD_SET(i, &Test);
 	    t.tv_sec = 0;
 	    t.tv_usec = 0;
-	    if (select(i + 1, (FDSET *)NULL, &Test, (FDSET *)NULL, &t) < 0
+	    if (select(i + 1, NULL, &Test, NULL, &t) < 0
 	     && errno != EINTR) {
 		syslog(L_ERROR, "%s Bad Write File %d", LogName, i);
 		FD_CLR(i, &WCHANmask);
@@ -807,7 +877,8 @@ CHANdiagnose(void)
 }
 #endif	/* defined(INND_FIND_BAD_FDS) */
 
-void CHANsetActiveCnx(CHANNEL *cp) {
+void
+CHANsetActiveCnx(CHANNEL *cp) {
     int		found;  
     CHANNEL	*tempchan;
     char	*label, *tmplabel;
@@ -839,25 +910,26 @@ void CHANsetActiveCnx(CHANNEL *cp) {
 **
 **  Yes, the main code has really wandered over to the side a lot.
 */
-void CHANreadloop(void)
+void
+CHANreadloop(void)
 {
     static char		EXITING[] = "INND exiting because of signal\n";
     static int		fd;
-    int			i;
+    int			i, j;
     int			startpoint;
     int			count;
     int			lastfd;
     int			oerrno;
     CHANNEL		*cp;
-    BUFFER		*bp;
-    FDSET		MyRead;
-    FDSET		MyWrite;
+    struct buffer	*bp;
+    fd_set		MyRead;
+    fd_set		MyWrite;
     struct timeval	MyTime;
     long		silence;
     char		*p;
     time_t		LastUpdate;
+    HDRCONTENT		*hc;
 
-    TMRinit();
     STATUSinit();
     
     LastUpdate = GetTimeInfo(&Now) < 0 ? 0 : Now.time;
@@ -869,16 +941,25 @@ void CHANreadloop(void)
 	MyRead = RCHANmask;
 	MyWrite = WCHANmask;
 	MyTime = TimeOut;
-        i = TMRmainloophook();
-        if (i != 0) MyTime.tv_sec = i;
+	if (innconf->timer) {
+	    unsigned long now = TMRnow();
+
+	    if (now >= 1000 * (unsigned long)(innconf->timer)) {
+		TMRsummary("ME", timer_name);
+		InndHisLogStats();
+		MyTime.tv_sec = innconf->timer;
+	    }
+	    else {
+		MyTime.tv_sec = innconf->timer - now / 1000;
+	    }
+	}
 	TMRstart(TMR_IDLE);
-	count = select(CHANlastfd + 1, &MyRead, &MyWrite, (FDSET *)NULL,
-		&MyTime);
+	count = select(CHANlastfd + 1, &MyRead, &MyWrite, NULL, &MyTime);
 	TMRstop(TMR_IDLE);
 
 	STATUSmainloophook();
 	if (GotTerminate) {
-	    (void)write(2, EXITING, STRLEN(EXITING));
+	    write(2, EXITING, strlen(EXITING));
 	    CleanupAndExit(0, (char *)NULL);
 	}
 	if (count < 0) {
@@ -895,7 +976,7 @@ void CHANreadloop(void)
 	if (GetTimeInfo(&Now) < 0)
 	    syslog(L_ERROR, "%s cant gettimeinfo %m", LogName);
 	if (Now.time > LastUpdate + TimeOut.tv_sec) {
-	    HISsync();
+	    HISsync(History);
 	    if (ICDactivedirty) {
 		ICDwriteactive();
 		ICDactivedirty = 0;
@@ -915,18 +996,22 @@ void CHANreadloop(void)
 	/* Try the control channel first. */
 	if (FD_ISSET(CHANccfd, &RCHANmask) && FD_ISSET(CHANccfd, &MyRead)) {
 	    count--;
- 	    if (count > 3) count = 3; /* might be more requests */
+ 	    if (count > 3)
+		count = 3; /* might be more requests */
 	    (*CHANcc->Reader)(CHANcc);
 	    FD_CLR(CHANccfd, &MyRead);
 	}
 
 #ifdef PRIORITISE_REMCONN
 	/* Try the remconn channel next. */
-	if (FD_ISSET(CHANrcfd, &RCHANmask) && FD_ISSET(CHANrcfd, &MyRead)) {
-	    count--;
-	    if (count > 3) count = 3; /* might be more requests */
-	    (*CHANrc->Reader)(CHANrc);
-	    FD_CLR(CHANrcfd, &MyRead);
+	for (j = 0 ; (j < chanlimit) && (CHANrcfd[j] >= 0) ; j++) {
+	    if (FD_ISSET(CHANrcfd[j], &RCHANmask) && FD_ISSET(CHANrcfd[j], &MyRead)) {
+		count--;
+		if (count > 3)
+		    count = 3; /* might be more requests */
+		(*CHANrc[j]->Reader)(CHANrc[j]);
+		FD_CLR(CHANrcfd[j], &MyRead);
+	    }
 	}
 #endif /* PRIORITISE_REMCONN */
 
@@ -970,7 +1055,7 @@ void CHANreadloop(void)
 			CHANname(cp), fd);
 		    /* Don't call RCHANremove since cp->fd will be -1. */
 		    FD_CLR(fd, &RCHANmask);
-		    (void)close(fd);
+		    close(fd);
 		}
 		else {
 		    cp->LastActive = Now.time;
@@ -980,16 +1065,53 @@ void CHANreadloop(void)
 
 	    /* Check and see if the buffer is grossly overallocated and shrink
 	       if needed */
-	    if (cp->In.Size > (BIG_BUFFER)) {
-		if (cp->In.Used) {
-		    if ((cp->In.Size / cp->In.Used) > 10) {
-			cp->In.Size = (cp->In.Used * 2) > START_BUFF_SIZE ? (cp->In.Used * 2) :  START_BUFF_SIZE;
-			cp->In.Data = RENEW(cp->In.Data, char, cp->In.Size);
-			cp->In.Left = cp->In.Size - cp->In.Used;
+	    if (cp->In.size > (BIG_BUFFER)) {
+		if (cp->In.used != 0) {
+		    if ((cp->In.size / cp->In.used) > 10) {
+			cp->In.size = (cp->In.used * 2) > START_BUFF_SIZE ? (cp->In.used * 2) : START_BUFF_SIZE;
+			p = cp->In.data;
+			TMRstart(TMR_DATAMOVE);
+                        cp->In.data = xrealloc(cp->In.data, cp->In.size);
+			cp->In.left = cp->In.size - cp->In.used;
+			/* do not move data, since xrealloc did it already */
+			if ((i = p - cp->In.data) != 0) {
+			    if (cp->State == CSgetheader ||
+				cp->State == CSgetbody ||
+				cp->State == CSeatarticle) {
+				/* adjust offset only in CSgetheader, CSgetbody
+				   or CSeatarticle */
+				if (cp->Data.BytesHeader != NULL)
+				  cp->Data.BytesHeader -= i;
+				hc = cp->Data.HdrContent;
+				for (j = 0 ; j < MAX_ARTHEADER ; j++, hc++) {
+				    if (hc->Value != NULL)
+					hc->Value -= i;
+				}
+			    }
+			}
+			TMRstop(TMR_DATAMOVE);
 		    }
 		} else {
-		    cp->In.Data = RENEW(cp->In.Data, char, START_BUFF_SIZE);
-		    cp->In.Size = cp->In.Left = START_BUFF_SIZE;
+		    p = cp->In.data;
+		    TMRstart(TMR_DATAMOVE);
+                    cp->In.data = xrealloc(cp->In.data, START_BUFF_SIZE);
+		    cp->In.size = cp->In.left = START_BUFF_SIZE;
+		    if ((i = p - cp->In.data) != 0) {
+			if (cp->State == CSgetheader ||
+			    cp->State == CSgetbody ||
+			    cp->State == CSeatarticle) {
+			    /* adjust offset only in CSgetheader, CSgetbody
+			       or CSeatarticle */
+			    if (cp->Data.BytesHeader != NULL)
+			      cp->Data.BytesHeader -= i;
+			    hc = cp->Data.HdrContent;
+			    for (j = 0 ; j < MAX_ARTHEADER ; j++, hc++) {
+				if (hc->Value != NULL)
+				    hc->Value -= i;
+			    }
+			}
+		    }
+		    TMRstop(TMR_DATAMOVE);
 		}
 	    }
 	    /* Possibly recheck for dead children so we don't get SIGPIPE
@@ -1005,21 +1127,21 @@ void CHANreadloop(void)
 			CHANname(cp), fd);
 		    /* Don't call WCHANremove since cp->fd will be -1. */
 		    FD_CLR(fd, &WCHANmask);
-		    (void)close(fd);
+		    close(fd);
 		}
 		else {
 		    bp = &cp->Out;
-		    if (bp->Left) {
+		    if (bp->left) {
 			cp->LastActive = Now.time;
-			i = CHANwrite(fd, &bp->Data[bp->Used], bp->Left);
+			i = CHANwrite(fd, &bp->data[bp->used], bp->left);
 			if (i <= 0) {
 			    oerrno = errno;
 			    p = CHANname(cp);
 			    errno = oerrno;
 			    if (i < 0)
-				syslog(L_ERROR, "%s cant write %m", p);
+				sysnotice("%s cant write", p);
 			    else
-				syslog(L_ERROR, "%s cant write", p);
+				notice("%s cant write", p);
 			    cp->BadWrites++;
 			    if (i < 0 && oerrno == EPIPE) {
 				SITEchanclose(cp);
@@ -1035,23 +1157,23 @@ void CHANreadloop(void)
 				syslog(L_ERROR, "%s sleeping", p);
 				WCHANremove(cp);
 				SCHANadd(cp,
-				    (time_t)(Now.time + innconf->pauseretrytime),
-				    (POINTER)NULL, CHANwakeup, (POINTER)NULL);
+                                         Now.time + innconf->pauseretrytime,
+                                         NULL, CHANwakeup, NULL);
 			    }
 			}
 			else {
 			    cp->BadWrites = 0;
 			    cp->BlockedWrites = 0;
-			    bp->Left -= i;
-			    bp->Used += i;
-			    if (bp->Left <= 0) {
+			    bp->left -= i;
+			    bp->used += i;
+			    if (bp->left <= 0) {
 				WCHANremove(cp);
 				(*cp->WriteDone)(cp);
-			    } else if (bp->Used > (bp->Size/COMP_THRESHOLD)) {
+			    } else if (bp->used > (bp->size/COMP_THRESHOLD)) {
                                 /* compact the buffer, shoving the
                                    data back to the beginning.
                                    <rmtodd@mailhost.ecn.ou.edu> */
-                                BUFFset(bp, &bp->Data[bp->Used], bp->Left);
+                                buffer_set(bp, &bp->data[bp->used], bp->left);
  			    }
 			}
 		    }
@@ -1066,7 +1188,7 @@ void CHANreadloop(void)
 		if (cp->Type == CTfree) {
 		    syslog(L_ERROR,"%s ERROR s-select free %d",CHANname(cp),fd);
 		    FD_CLR(fd, &SCHANmask);
-		    (void) close(fd);
+		     close(fd);
 		} else {
 		    cp->LastActive = Now.time;
 		    SCHANremove(cp);
