@@ -1,5 +1,4 @@
-/*
-**  $Id: python.c,v 1.5.2.2 2001/10/04 01:35:40 kondou Exp $
+/*  $Id: python.c,v 1.23 2003/02/11 06:48:55 alexk Exp $
 **
 **  python.c: Embed Python in the style of innd's TCL and Perl stuff.
 ** 
@@ -12,14 +11,11 @@
 **  against Python's OO model here.  Don't tell Guido.
 */
 
-#include <stdio.h>
-#include <sys/types.h>
-#include <netinet/in.h>
-#include "configdata.h"
+#include "config.h"
 #include "clibrary.h"
+
+#include "inn/innconf.h"
 #include "innd.h"
-#include "art.h"
-#include "qio.h"
 
 
 #if defined(DO_PYTHON)
@@ -27,13 +23,12 @@
 #include "Python.h"
 
 
-BOOL		PythonFilterActive;
+bool		PythonFilterActive;
 char		*filterPath;	/* this gets set in art.c */
 PyObject	*PYFilterObject = NULL;
 PyObject	*PYFilterModule = NULL;
 
 /* article filter bits and pieces */
-ARTHEADER	ARTheaders[], *ARTheadersENDOF;
 PyObject	*PYheaders = NULL;
 PyObject	**PYheaditem;
 PyObject	**PYheadkey;
@@ -53,7 +48,7 @@ PyObject	*close_method = NULL;
 */
 void
 PYfilter(value)
-    BOOL value;
+    bool value;
 {
     PythonFilterActive = value;
     syslog(L_NOTICE, "%s Python filtering %s", LogName,
@@ -65,12 +60,11 @@ PYfilter(value)
 /*
 **  Front end for PYfilter()
 */
-STRING
-PYcontrol(av)
-    char **av;
+const char *
+PYcontrol(char **av)
 {
     char        *p;
-    extern BOOL PythonFilterActive;
+    extern bool PythonFilterActive;
 
     switch (av[0][0]) {
     default:
@@ -80,12 +74,12 @@ PYcontrol(av)
             return "1 Python filter already enabled";
         else if (PYFilterObject == NULL)
             return "1 Python filter not defined" ;
-        PYfilter(TRUE);
+        PYfilter(true);
         break;
     case 'n':
         if (!PythonFilterActive)
             return "1 Python filter already disabled";
-        PYfilter(FALSE);
+        PYfilter(false);
         break;
     }
     return NULL;
@@ -98,12 +92,10 @@ PYcontrol(av)
 **  Reject articles we don't like.
 */
 char *
-PYartfilter(artBody, artLen, lines)
-    char* artBody;
-    long artLen;
-    int lines;
+PYartfilter(const ARTDATA *data, char *artBody, long artLen, int lines)
 {
-    ARTHEADER	*hp;
+    const ARTHEADER *hp;
+    const HDRCONTENT *hc = data->HdrContent;
     int		hdrnum;
     int		i;
     char	*p, save;
@@ -115,29 +107,16 @@ PYartfilter(artBody, artLen, lines)
 
     /* Add headers to the dictionary... */
     hdrnum = 0;
-    for (hp = ARTheaders; hp < ARTheadersENDOF; hp++) {
-	if (hp->Found && hp->Value && !EQ(hp->Name,"Path"))
-	    PYheaditem[hdrnum] = PyBuffer_FromMemory(hp->Value, hp->Length);
-	else
+    for (i = 0 ; i < MAX_ARTHEADER ; i++) {
+	if (HDR_FOUND(i)) {
+	    hp = &ARTheaders[i];
+	    PYheaditem[hdrnum] = PyBuffer_FromMemory(HDR(i), HDR_LEN(i));
+	} else
 	    PYheaditem[hdrnum] = Py_None;
 	PyDict_SetItem(PYheaders, PYheadkey[hdrnum], PYheaditem[hdrnum]);
 	hdrnum++;
     }
 
-    /* ...then the path, done separately to get the info innd adds... */
-    if (filterPath != NULL) {
-	p = strpbrk(filterPath,"\r\n");
-	if (p) {
-	    save = *p;
-	    *p = '\0';
-	}
-	PYheaditem[hdrnum] = PyBuffer_FromMemory(filterPath,
-						 strlen(filterPath));
-	PyDict_SetItem(PYheaders, PYpathkey, PYheaditem[hdrnum++]);
-	if (p)
-	    *p = save;
-    }
-
     /* ...then the body... */
     if (artLen && artBody != NULL)
         PYheaditem[hdrnum] = PyBuffer_FromMemory(artBody, --artLen);
@@ -152,7 +131,7 @@ PYartfilter(artBody, artLen, lines)
     /* Now see if the filter likes it. */
     result = PyObject_CallFunction(art_method, "O", PYheaders);
     if ((result != NULL) && PyObject_IsTrue(result))
-	strncpy(buf, PyString_AS_STRING(result), sizeof(buf) - 1);
+	strlcpy(buf, PyString_AS_STRING(result), sizeof(buf));
     else
 	*buf = '\0';
     Py_XDECREF(result);
@@ -186,7 +165,7 @@ PYmidfilter(messageID, msglen)
 
     result = PyObject_CallFunction(msgid_method, "s#", messageID, msglen);
     if ((result != NULL) && PyObject_IsTrue(result))
-	strncpy(buf,  PyString_AS_STRING(result), sizeof(buf) - 1);
+	strlcpy(buf, PyString_AS_STRING(result), sizeof(buf));
     else
 	*buf = '\0';
     Py_XDECREF(result);
@@ -213,17 +192,17 @@ PYmode(Mode, NewMode, reason)
 	return;
 
     switch (Mode) {
-    default:		strcpy(oldmode, "unknown");	break;
-    case OMrunning:	strcpy(oldmode, "running");	break;
-    case OMpaused:	strcpy(oldmode, "paused");	break;
-    case OMthrottled:	strcpy(oldmode, "throttled");	break;
+    default:		strlcpy(oldmode, "unknown", 10);	break;
+    case OMrunning:	strlcpy(oldmode, "running", 10);	break;
+    case OMpaused:	strlcpy(oldmode, "paused", 10);		break;
+    case OMthrottled:	strlcpy(oldmode, "throttled", 10);	break;
     }
 
     switch (NewMode) {
-    default:		strcpy(newmode, "unknown");	break;
-    case OMrunning:	strcpy(newmode, "running");	break;
-    case OMpaused:	strcpy(newmode, "paused");	break;
-    case OMthrottled:	strcpy(newmode, "throttled");	break;
+    default:		strlcpy(newmode, "unknown", 10);	break;
+    case OMrunning:	strlcpy(newmode, "running", 10);	break;
+    case OMpaused:	strlcpy(newmode, "paused", 10);		break;
+    case OMthrottled:	strlcpy(newmode, "throttled", 10);	break;
     }
 
     result = PyObject_CallFunction(mode_method, "sss",
@@ -268,7 +247,7 @@ PY_havehist(self, args)
     if (!PyArg_ParseTuple(args, "s#", &msgid, &msgidlen))
 	return NULL;
 
-    if (HIShavearticle(HashMessageID(msgid)))
+    if (HIScheck(History, msgid))
 	return PyInt_FromLong(1);
     return PyInt_FromLong(0);
 }
@@ -315,7 +294,7 @@ PY_addhist(self, args)
     if (!PyArg_ParseTuple(args, "s#", &msgid, &msgidlen))
 	return NULL;
 
-    sprintf(tbuff, "%d",time((long *)0));
+    snprintf(tbuff, sizeof(tbuff), "%d", time(NULL));
 
     parambuf[0] = msgid;
     parambuf[1] = parambuf[2] = parambuf[3] = tbuff;
@@ -380,7 +359,7 @@ PY_head(self, args)
     char	*msgid;
     int		msgidlen;
     char	*p;
-    TOKEN	*token;
+    TOKEN	token;
     ARTHANDLE	*art;
     PyObject	*header;
     int		headerlen;
@@ -388,15 +367,14 @@ PY_head(self, args)
     if (!PyArg_ParseTuple(args, "s#", &msgid, &msgidlen))
 	return NULL;
 
-    /* Get the article filenames/token; open the first */
-    if ((token = HISfilesfor(HashMessageID(msgid))) == NULL)
+    if (! HISlookup(History, msgid, NULL, NULL, NULL, &token))
 	return Py_BuildValue("s", "");	
-    if ((art = SMretrieve(*token, RETR_HEAD)) == NULL)
+    if ((art = SMretrieve(token, RETR_HEAD)) == NULL)
 	return Py_BuildValue("s", "");	
     p = FromWireFmt(art->data, art->len, &headerlen);
     SMfreearticle(art);
     header = PyString_FromStringAndSize(p, headerlen);
-    DISPOSE(p);
+    free(p);
 
     return header;
 }
@@ -413,7 +391,7 @@ PY_article(self, args)
     char	*msgid;
     int		msgidlen;
     char	*p;
-    TOKEN	*token;
+    TOKEN	token;
     ARTHANDLE	*arth;
     PyObject	*art;
     int		artlen;
@@ -421,15 +399,14 @@ PY_article(self, args)
     if (!PyArg_ParseTuple(args, "s#", &msgid, &msgidlen))
 	return NULL;
 
-    /* Get the article filenames; open the first file */
-    if ((token = HISfilesfor(HashMessageID(msgid))) == NULL)
+    if (! HISlookup(History, msgid, NULL, NULL, NULL, &token))
 	return Py_BuildValue("s", "");
-    if ((arth = SMretrieve(*token, RETR_ALL)) == NULL)
+    if ((arth = SMretrieve(token, RETR_ALL)) == NULL)
 	return Py_BuildValue("s", "");	
     p = FromWireFmt(arth->data, arth->len, &artlen);
     SMfreearticle(arth);
     art = PyString_FromStringAndSize(p, artlen);
-    DISPOSE(p);
+    free(p);
 
     return art;
 }
@@ -521,7 +498,7 @@ PY_hashstring(self, args)
 	/* Compress out multiple whitespace in the trimmed string.  We
 	 * do a copy because this is probably an original art
 	 * buffer. */
-	workstring =  memcpy(NEW(char, worksize), wpos, worksize);
+	workstring =  memcpy(xmalloc(worksize), wpos, worksize);
 	newsize = wasspace = 0;
 	p = wpos;
 	q = workstring;
@@ -539,7 +516,7 @@ PY_hashstring(self, args)
 	}
 	worksize = q - workstring;
 	myhash = Hash(workstring, worksize);
-	DISPOSE(workstring);
+	free(workstring);
     }
     else
 	myhash = Hash(instring, insize);
@@ -643,17 +620,17 @@ PYreadfilter(void)
 
     if ((newmodule = PyImport_ReloadModule(PYFilterModule)) == NULL) {
 	syslog(L_ERROR, "cant reload python filter module");
-	PYfilter(FALSE);
+	PYfilter(false);
 	return 0;
     }
     Py_DECREF(PYFilterModule);
     PYFilterModule = newmodule;
     if (PYFilterObject == NULL) {
 	syslog(L_ERROR, "python reload error, filter object not defined");
-	PYfilter(FALSE);
+	PYfilter(false);
 	return 0;
     }
-    PYfilter(TRUE);
+    PYfilter(true);
     PYdefmethods();
     return 1;
 }
@@ -666,7 +643,7 @@ PYreadfilter(void)
 void
 PYsetup(void)
 {
-    ARTHEADER	*hp;
+    const ARTHEADER *hp;
     int		hdrindex;
 
     setenv("PYTHONPATH", innconf->pathfilter, 1);
@@ -683,7 +660,7 @@ PYsetup(void)
     }
     syslog(L_NOTICE, "python interpreter initialized OK");
 
-    (void) Py_InitModule("INN", INNPyMethods);
+    Py_InitModule("INN", INNPyMethods);
 
     PYFilterModule = PyImport_ImportModule(_PATH_PYTHON_STARTUP_M);
     if (PYFilterModule == NULL)
@@ -691,20 +668,20 @@ PYsetup(void)
 
     if (PYFilterObject == NULL) {
 	syslog(L_ERROR, "python filter object is not defined");
-	PYfilter(FALSE);
+	PYfilter(false);
     } else {
-	PYfilter(TRUE);
+	PYfilter(true);
 	PYdefmethods();
 	syslog(L_NOTICE, "defined python methods");
     }
 
     /* Grab space for these so we aren't forever recreating them. */
     PYheaders = PyDict_New();
-    PYheaditem = NEW(PyObject *, ARTheadersENDOF - ARTheaders);
-    PYheadkey = NEW(PyObject *, ARTheadersENDOF - ARTheaders);
+    PYheaditem = xmalloc((ARRAY_END(ARTheaders) - ARTheaders) * sizeof(PyObject *));
+    PYheadkey = xmalloc((ARRAY_END(ARTheaders) - ARTheaders) * sizeof(PyObject *));
 
     /* Preallocate keys for the article dictionary */
-    for (hp = ARTheaders; hp < ARTheadersENDOF; hp++)
+    for (hp = ARTheaders; hp < ARRAY_END(ARTheaders); hp++)
 	PYheadkey[hp - ARTheaders] = PyString_InternFromString(hp->Name);
     PYpathkey = PyString_InternFromString("Path");
     PYlineskey = PyString_InternFromString("__LINES__");
