@@ -1,38 +1,22 @@
-/* -*- c -*-
- *
- * Author:      James Brister <brister@vix.com> -- berkeley-unix --
- * Start Date:  Wed Dec 27 10:09:28 1995
- * Project:     INN (innfeed)
- * File:        article.h
- * RCSId:       $Id: article.h,v 1.2 1999/03/25 10:03:41 kondou Exp $
- * Copyright:   Copyright (c) 1996 by Internet Software Consortium
- *
- *              Permission to use, copy, modify, and distribute this
- *              software for any purpose with or without fee is hereby
- *              granted, provided that the above copyright notice and this
- *              permission notice appear in all copies.
- *
- *              THE SOFTWARE IS PROVIDED "AS IS" AND INTERNET SOFTWARE
- *              CONSORTIUM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
- *              SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
- *              MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL INTERNET
- *              SOFTWARE CONSORTIUM BE LIABLE FOR ANY SPECIAL, DIRECT,
- *              INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
- *              WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
- *              WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
- *              TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE
- *              USE OR PERFORMANCE OF THIS SOFTWARE.
- * Description: The public interface to articles. The articles are
- *              implemented via reference counting. This interface
- *              provides the methods for getting the contents of the article.
- *
- *              When an Article is created there's a chance that another
- *              copy of it already exists. For example if the Article is
- *              pulled out of a Tape for a particular host it may already
- *              be in existance in some other host. This class will manage
- *              this situation to prevent multiple copies of the article
- *              being in core.
- */
+/*  $Id: article.h,v 1.4 2002/10/07 07:36:10 rra Exp $
+**
+**  The public interface to articles.
+**
+**  Written by James Brister <brister@vix.com>
+**  Copyright 1996 by the Internet Software Consortium
+**
+**  For license terms, see the end of this file.
+**
+**  The public interface to articles. The articles are implemented via
+**  reference counting.  This interface provides the methods for getting the
+**  contents of the article.
+**
+**  When an Article is created there's a chance that another copy of it
+**  already exists.  For example if the Article is pulled out of a Tape for a
+**  particular host it may already be in existance in some other host.  This
+**  class will manage this situation to prevent multiple copies of the article
+**  being in core.
+*/
 
 #if ! defined ( article_h__ )
 #define article_h__
@@ -50,10 +34,10 @@ Article newArticle (const char *filename, const char *msgid) ;
      refcount is 0. */
 void delArticle (Article article) ;
 
-void gPrintArticleInfo (FILE *fp, u_int indentAmt) ;
+void gPrintArticleInfo (FILE *fp, unsigned int indentAmt) ;
 
   /* print some debugging info about the article. */
-void printArticleInfo (Article art, FILE *fp, u_int indentAmt) ;
+void printArticleInfo (Article art, FILE *fp, unsigned int indentAmt) ;
 
   /* return true if this article's file still exists. */
 bool artFileIsValid (Article article) ;
@@ -80,7 +64,7 @@ const char *artMsgId (Article article) ;
 int artSize (Article article) ;
 
   /* return the number of buffers that artGetNntpBuffers() would return. */
-u_int artNntpBufferCount (Article article) ;
+unsigned int artNntpBufferCount (Article article) ;
 
   /* tell the Article class to log (or not) missing articles as they occur. */
 void artLogMissingArticles (bool val) ;
@@ -94,6 +78,23 @@ void artBitFiddleContents (bool val) ;
 
   /* set the limit on the number of bytes in all articles (this is not a hard
      limit). Can only be called one time before any articles are created. */
-void artSetMaxBytesInUse (u_int val) ;
+void artSetMaxBytesInUse (unsigned int val) ;
 
 #endif /* article_h__ */
+
+/*
+**  Copyright 1996 by the Internet Software Consortium
+**
+**  Permission to use, copy, modify, and distribute this software for any
+**  purpose with or without fee is hereby granted, provided that the above
+**  copyright notice and this permission notice appear in all copies.
+**
+**  THE SOFTWARE IS PROVIDED "AS IS" AND INTERNET SOFTWARE CONSORTIUM
+**  DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL
+**  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.  IN NO EVENT SHALL
+**  INTERNET SOFTWARE CONSORTIUM BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
+**  OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
+**  USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
+**  OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
+**  PERFORMANCE OF THIS SOFTWARE.
+*/
