@@ -1,39 +1,12 @@
-/* -*- c -*-
- *
- * Author:      James Brister <brister@vix.com> -- berkeley-unix --
- * Start Date:  Thu Dec 28 13:15:04 1995
- * Project:     INN (innfeed)
- * File:        innlistener.c
- * RCSId:       $Id: innlistener.c,v 1.16.2.1 2000/06/08 10:07:20 rra Exp $
- *
- * Copyright:   Copyright (c) 1996 by Internet Software Consortium
- *
- *              Permission to use, copy, modify, and distribute this
- *              software for any purpose with or without fee is hereby
- *              granted, provided that the above copyright notice and this
- *              permission notice appear in all copies.
- *
- *              THE SOFTWARE IS PROVIDED "AS IS" AND INTERNET SOFTWARE
- *              CONSORTIUM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
- *              SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
- *              MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL INTERNET
- *              SOFTWARE CONSORTIUM BE LIABLE FOR ANY SPECIAL, DIRECT,
- *              INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
- *              WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
- *              WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
- *              TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE
- *              USE OR PERFORMANCE OF THIS SOFTWARE.
- *
- * Description: Implementation of the InnListener class.
- * 
- */
-
-#if ! defined (lint)
-static const char *rcsid = "$Id: innlistener.c,v 1.16.2.1 2000/06/08 10:07:20 rra Exp $" ;
-static void use_rcsid (const char *rid) {   /* Never called */
-  use_rcsid (rcsid) ; use_rcsid (rid) ;
-}
-#endif
+/*  $Id: innlistener.c,v 1.30 2003/01/19 19:58:22 rra Exp $
+**
+**  The implementation of the innfeed InnListener class.
+**
+**  Written by James Brister <brister@vix.com>
+**  Copyright 1996 by the Internet Software Consortium
+**
+**  For license terms, see the end of this file.
+*/
 
 #include "innfeed.h"
 #include "config.h"
@@ -42,12 +15,12 @@ static void use_rcsid (const char *rid) {   /* Never called */
 #include <assert.h>
 #include <ctype.h>
 #include <errno.h>
+#include <fcntl.h>
 #include <syslog.h>
 #include <time.h>
 
-#ifdef HAVE_FCNTL_H
-# include <fcntl.h>
-#endif
+#include "inn/messages.h"
+#include "libinn.h"
 
 #include "article.h"
 #include "buffer.h"
@@ -55,7 +28,6 @@ static void use_rcsid (const char *rid) {   /* Never called */
 #include "endpoint.h"
 #include "host.h"
 #include "innlistener.h"
-#include "msgs.h"
 #include "nntp.h"
 #include "tape.h"
 
@@ -76,7 +48,7 @@ struct innlistener_s
     InnListener next ;
 };
 
-static u_int listenerCount = 0 ;
+static unsigned int listenerCount = 0 ;
 static InnListener listenerList = NULL ;
 
 InnListener mainListener ;
@@ -108,7 +80,7 @@ static bool inited = false ;
 void listenerLogStatus (FILE *fp)
 {
   fprintf (fp,"%sListener Status:%s\n",
-           genHtml ? "<H2>" : "", genHtml ? "</H2>" : "") ;
+           genHtml ? "<B>" : "", genHtml ? "</B>" : "") ;
   fprintf (fp,"    Dropped article file: %s\n",dropArtFile) ;
   fprintf (fp,"   Dropped article count: %ld\n",(long) droppedCount) ;
   fprintf (fp,"\n") ;
@@ -116,7 +88,7 @@ void listenerLogStatus (FILE *fp)
 
 InnListener newListener (EndPoint endp, bool isDummy, bool dynamicPeers)
 {
-  InnListener l = CALLOC (struct innlistener_s, 1) ;
+  InnListener l = xcalloc (1, sizeof(struct innlistener_s)) ;
   Buffer *readArray ;
 
   if (!inited)
@@ -125,13 +97,10 @@ InnListener newListener (EndPoint endp, bool isDummy, bool dynamicPeers)
       atexit (listenerCleanup) ;
     }
   
-  ASSERT (l != NULL) ;
-
   l->myep = endp ;
 
   l->hostLen = MAX_HOSTS ;
-  l->myHosts = CALLOC (Host, l->hostLen) ;
-  ASSERT (l->myHosts != NULL) ;
+  l->myHosts = xcalloc (l->hostLen, sizeof(Host)) ;
 
   l->inputBuffer = newBuffer (LISTENER_INPUT_BUFFER) ;
   l->dummyListener = isDummy ;
@@ -152,18 +121,18 @@ InnListener newListener (EndPoint endp, bool isDummy, bool dynamicPeers)
   return l ;
 }
 
-void gPrintListenerInfo (FILE *fp, u_int indentAmt)
+void gPrintListenerInfo (FILE *fp, unsigned int indentAmt)
 {
   InnListener p ;
   char indent [INDENT_BUFFER_SIZE] ;
-  u_int i ;
+  unsigned int i ;
   
   for (i = 0 ; i < MIN(INDENT_BUFFER_SIZE - 1,indentAmt) ; i++)
     indent [i] = ' ' ;
   indent [i] = '\0' ;
 
   fprintf (fp,"%sGlobal InnListener list : %p (count %d) {\n",
-           indent,listenerList,listenerCount) ;
+           indent,(void *) listenerList,listenerCount) ;
   for (p = listenerList ; p != NULL ; p = p->next)
     printListenerInfo (p,fp,indentAmt + INDENT_INCR) ;
   fprintf (fp,"%s}\n",indent) ;
@@ -171,17 +140,17 @@ void gPrintListenerInfo (FILE *fp, u_int indentAmt)
 
 
 
-void printListenerInfo (InnListener listener, FILE *fp, u_int indentAmt)
+void printListenerInfo (InnListener listener, FILE *fp, unsigned int indentAmt)
 {
   char indent [INDENT_BUFFER_SIZE] ;
-  u_int i ;
+  unsigned int i ;
   
   for (i = 0 ; i < MIN(INDENT_BUFFER_SIZE - 1,indentAmt) ; i++)
     indent [i] = ' ' ;
   indent [i] = '\0' ;
 
-  fprintf (fp,"%sInnListener : %p {\n",indent,listener) ;
-  fprintf (fp,"%s    endpoint : %p\n", indent, listener->myep) ;
+  fprintf (fp,"%sInnListener : %p {\n",indent,(void *) listener) ;
+  fprintf (fp,"%s    endpoint : %p\n", indent,(void *) listener->myep) ;
   fprintf (fp,"%s    dummy-listener : %s\n",indent,
            boolToString (listener->dummyListener)) ;
   fprintf (fp,"%s    dynamicPeers : %s\n",indent,
@@ -198,7 +167,7 @@ void printListenerInfo (InnListener listener, FILE *fp, u_int indentAmt)
       if (listener->myHosts [i] != NULL)
         printHostInfo (listener->myHosts [i],fp,indentAmt + INDENT_INCR) ;
 #else
-      fprintf (fp,"%s        %p\n",indent,listener->myHosts[i]) ;
+      fprintf (fp,"%s        %p\n",indent,(void *) listener->myHosts[i]) ;
 #endif
     }
   
@@ -226,8 +195,8 @@ static void unlinkPidFile (void)
      Listener will be deleted. */
 void shutDown (InnListener l)
 {
-  u_int i ;
-  u_int count ;
+  unsigned int i ;
+  unsigned int count ;
 
   d_printf (1,"Shutting down the listener\n") ;
 
@@ -264,13 +233,13 @@ void shutDown (InnListener l)
       char dateString [30] ;
 
       gHostStats();
-      strcpy (dateString,ctime (&now)) ;
+      strlcpy (dateString,ctime (&now),sizeof (dateString)) ;
       dateString [24] = '\0' ;
 
       if (fastExit)
-        syslog (LOG_NOTICE,FAST_EXIT_PROGRAM,dateString) ;
+        notice ("ME finishing (quickly) at %s", dateString) ;
       else
-        syslog (LOG_NOTICE,STOPPING_PROGRAM,dateString) ;
+        notice ("ME finishing at %s", dateString) ;
 
       unlinkPidFile();
       exit (0) ;
@@ -280,7 +249,7 @@ void shutDown (InnListener l)
 
 bool listenerAddPeer (InnListener listener, Host hostObj)
 {
-  u_int i ;
+  unsigned int i ;
 
   d_printf (1,"Adding peer: %s\n", hostPeerName (hostObj)) ;
   
@@ -305,10 +274,10 @@ bool listenerIsDummy (InnListener listener)
 }
 
 /* Called by the Host when it (the Host) is about to delete itself. */
-u_int listenerHostGone (InnListener listener, Host host)
+unsigned int listenerHostGone (InnListener listener, Host host)
 {
-  u_int i ;
-  u_int someThere = 0 ;
+  unsigned int i ;
+  unsigned int someThere = 0 ;
 
   d_printf (1,"Host is gone: %s\n", hostPeerName (host)) ;
   
@@ -358,7 +327,7 @@ void openInputFile (void)
     {
       if (dup2(fd, mainFd) < 0)
 	die ("dup2 %d %d: %s\n", fd, mainFd, strerror(errno)) ;
-      (void) close (fd);
+      close (fd);
     }
 
   i = read(mainFd, buf, sizeof (buf)) ;
@@ -372,18 +341,16 @@ void openInputFile (void)
       if (offset > 0 && *p == '\n')
 	lseek (mainFd, offset, SEEK_SET) ;
       else
-	lseek (mainFd, (OFFSET_T)0, SEEK_SET) ;
+	lseek (mainFd, 0, SEEK_SET) ;
     }
   syslog(LOG_NOTICE, "ME opened %s", InputFile);
 }
 
 
-int listenerConfigLoadCbk (void *data)
+int listenerConfigLoadCbk (void *data UNUSED)
 {
   int bval ;
 
-  (void) data ;
-
   if (getBool (topScope,"fast-exit",&bval,NO_INHERIT))
     fastExit = (bval ? true : false) ;
 
@@ -437,7 +404,7 @@ static void newArticleCommand (EndPoint ep, IoStatus i,
       else 
         {
           d_printf (1,"Got EOF on listener\n") ;
-          syslog (LOG_NOTICE,INN_GONE) ;
+          notice ("ME source lost . Exiting");
 	  shutDown (lis) ;
         }
     }
@@ -447,7 +414,7 @@ static void newArticleCommand (EndPoint ep, IoStatus i,
 #if HAVE_SOCKETPAIR
       if (errno != ECONNABORTED)
 #endif
-      syslog (LOG_CRIT,INN_IO_ERROR) ;
+      syswarn ("ME source read error, exiting") ;
       d_printf (1,"Got IO Error on listener\n") ;
       shutDown (lis) ;
     }
@@ -457,21 +424,21 @@ static void newArticleCommand (EndPoint ep, IoStatus i,
          precede the newline, we never get out of here */
       if (strlen(bbase) < blen)
         {
-          syslog (LOG_CRIT,INN_BAD_CMD,bbase) ;
+          warn ("ME source format bad, exiting: %s", bbase) ;
           shutDown (lis) ;
 
           return ;
         }
       if (blen == bufferSize(buffs [0])) {
         if (!expandBuffer (buffs [0], BUFFER_EXPAND_AMOUNT)) {
-          syslog (LOG_CRIT,L_BUFFER_EXPAND_ERROR);
+          warn ("ME error expanding input buffer") ;
           shutDown (lis) ;
           return ;
         }
       }
       readArray = makeBufferArray (bufferTakeRef (buffs [0]),NULL) ;
       if (!prepareRead (ep, readArray, newArticleCommand, data, 1)) {
-        syslog (LOG_CRIT,L_PREPARE_READ_FAILED) ;
+        warn ("ME error prepare read failed") ;
         freeBufferArray (readArray) ;
         shutDown (lis) ;
         return ;
@@ -499,7 +466,7 @@ static void newArticleCommand (EndPoint ep, IoStatus i,
 	     by newline, we may skip a large chunk without noticing */
 	  if (*next == '\0' && next < bbase + blen)
             {
-              syslog (LOG_CRIT,INN_BAD_CMD,cmd) ;
+              warn ("ME source format bad, exiting: %s", cmd) ;
               shutDown (lis) ;
 
               return ;
@@ -510,7 +477,7 @@ static void newArticleCommand (EndPoint ep, IoStatus i,
           /* pick out the leading string (the filename) */
           if ((fileName = findNonBlankString (cmd,&fileNameEnd)) == NULL)
             {
-              syslog (LOG_CRIT,INN_BAD_CMD,cmd) ;
+              warn ("ME source format bad, exiting: %s", cmd) ;
               shutDown (lis) ;
 
               return ;
@@ -522,7 +489,7 @@ static void newArticleCommand (EndPoint ep, IoStatus i,
           if ((msgid = findNonBlankString (fileNameEnd + 1,&msgidEnd)) == NULL)
             {
               *fileNameEnd = ' ' ; /* to make syslog work properly */
-              syslog (LOG_CRIT,INN_BAD_CMD,cmd) ;
+              warn ("ME source format bad, exiting: %s", cmd) ;
               shutDown (lis) ;
 
               return ;
@@ -537,14 +504,15 @@ static void newArticleCommand (EndPoint ep, IoStatus i,
 
           /* Check the message ID length */
           if (strlen(msgid) > NNTP_MSGID_MAXLEN) {
-            syslog(LOG_ERR,INN_MSGID_SIZE,NNTP_MSGID_MAXLEN,msgid) ;
+            warn ("ME message id exceeds limit of %d octets: %s",
+                  NNTP_MSGID_MAXLEN, msgid) ;
             *(msgidEnd+1) = '\0' ;
           }
           *msgidEnd = ' ' ;
 
           /* Check if message ID starts with < and ends with > */
           if (*msgid != '<' || *(msgidEnd-1) != '>') {
-            syslog(LOG_ERR,INN_BAD_CMD,cmd);
+            warn ("ME source format bad, exiting: %s", cmd) ;
             *(msgidEnd+1) = '\0';
           }
 
@@ -562,10 +530,10 @@ static void newArticleCommand (EndPoint ep, IoStatus i,
               
               /* See if this is a valid peername */
               for(s = peer; *s; s++)
-                if (!isalnum(*s) && *s != '.' && *s != '-' && *s != '_')
+                if (!CTYPE(isalnum, *s) && *s != '.' && *s != '-' && *s != '_')
                   break;
               if (*s != 0) {
-                  syslog(LOG_ERR,INVALID_PEER,peer);
+                  warn ("ME invalid peername %s", peer) ;
                   continue;
               }
               if (article != NULL)
@@ -591,7 +559,7 @@ static void newArticleCommand (EndPoint ep, IoStatus i,
       if (*cmd != '\0')         /* partial command left in buffer */
         {
           Buffer *bArr ;
-          u_int leftAmt = blen - (cmd - bbase) ;
+          unsigned int leftAmt = blen - (cmd - bbase) ;
 
           ASSERT (cmd != bbase) ;
           /* first we shift whats left in the buffer down to the bottom */
@@ -602,7 +570,7 @@ static void newArticleCommand (EndPoint ep, IoStatus i,
       
           if ( !prepareRead (lis->myep, bArr, newArticleCommand, lis, 1) )
             {
-              syslog (LOG_CRIT,L_PREPARE_READ_FAILED) ;
+              warn ("ME error prepare read failed") ;
 
               freeBufferArray (bArr) ;
               
@@ -619,7 +587,7 @@ static void newArticleCommand (EndPoint ep, IoStatus i,
       
           if ( !prepareRead (lis->myep, bArr, newArticleCommand, lis, 1) )
             {
-              syslog (LOG_CRIT,L_PREPARE_READ_FAILED) ;
+              warn ("ME error prepare read failed") ;
 
               shutDown (lis) ;
 
@@ -651,7 +619,7 @@ static void wakeUp (TimeoutId id, void *data)
 static void giveArticleToPeer (InnListener lis,
                                Article article, const char *peerName)
 {
-  u_int i ;
+  unsigned int i ;
 
   for (i = 0 ; i < lis->hostLen ; i++)
     if (lis->myHosts[i] != NULL)
@@ -685,7 +653,7 @@ static void giveArticleToPeer (InnListener lis,
             {
               /* XXX need to remember we've gone over the limit and not try
                  to add any more. */
-              syslog (LOG_ERR, TOO_MANY_HOSTS, lis->hostLen) ;
+              warn ("ME internal too many hosts. (max is %d)", lis->hostLen) ;
               dropArticle (peerName,article) ;
             }
           else
@@ -709,14 +677,14 @@ static void writeCheckPoint (int offsetAdjust)
   int writeBytes, writeReturn, mainFd ;
 	      
   mainFd = getMainEndPointFd() ;
-  offset = lseek (mainFd, (OFFSET_T)0, SEEK_CUR) ;
+  offset = lseek (mainFd, 0, SEEK_CUR) ;
   if (offset < 0)
     syslog (LOG_ERR, "ME tell(mainFd): %m") ;
   else
     {
-      (void) sprintf (offsetString, "%ld\n",
+      snprintf (offsetString, sizeof(offsetString), "%ld\n",
 		      (long)(offset - offsetAdjust) ) ;
-      if ( lseek (mainFd, (OFFSET_T)0, SEEK_SET) != 0 )
+      if ( lseek (mainFd, 0, SEEK_SET) != 0 )
 	syslog (LOG_ERR, "ME seek(mainFd, 0, 0): %m") ;
       else
 	{
@@ -745,26 +713,26 @@ void openDroppedArticleFile (void)
 {
   pid_t myPid = getpid () ;
   const char *tapeDir = getTapeDirectory() ;
+  size_t len;
 
   if (dropArtFile != NULL)
-    FREE (dropArtFile) ;
-  
-  dropArtFile = malloc (pathMax(tapeDir) + 1) ;
-  
-  sprintf (dropArtFile,"%s/innfeed-dropped.%c%06d",
-           tapeDir, droppedFileCount + 'A', (int) myPid) ;
+    free (dropArtFile) ;
+
+  len = pathMax(tapeDir) + 1;
+  dropArtFile = xmalloc(len);
+  snprintf (dropArtFile,len,"%s/innfeed-dropped.%c%06d",
+            tapeDir, droppedFileCount + 'A', (int) myPid) ;
 
   if ((droppedFp = fopen (dropArtFile,"w")) == NULL)
     {
-      syslog (LOG_ERR,NO_DROPPED_FILE,dropArtFile) ;
+      syswarn ("ME cant open %s: loosing articles", dropArtFile) ;
 
-      FREE (dropArtFile) ;
+      free (dropArtFile) ;
       dropArtFile = NULL ;
       
       if ((droppedFp = fopen ("/dev/null","w")) == NULL)
         {
-          syslog (LOG_ERR,NO_NULL_FILE) ;
-          die ("Error opening /dev/null") ;
+          die ("ME error opening /dev/null") ;
         }
     }
 
@@ -773,13 +741,13 @@ void openDroppedArticleFile (void)
 
 void closeDroppedArticleFile (void)
 {
-  long pos ;
+  off_t pos ;
 
   if (droppedFp == NULL)
     return ;
 
   fflush (droppedFp) ;
-  pos = ftell (droppedFp) ;
+  pos = ftello (droppedFp) ;
 
   fclose (droppedFp) ;
   droppedFp = NULL ;
@@ -787,9 +755,9 @@ void closeDroppedArticleFile (void)
   if (pos == 0 && dropArtFile != NULL)
     unlink (dropArtFile) ;
   else if (pos != 0 && dropArtFile == NULL)
-    syslog (LOG_WARNING,LOST_ARTICLE_COUNT,droppedCount) ;
+    warn ("ME lost %ld articles", droppedCount) ;
   else if (pos != 0)
-    syslog (LOG_NOTICE,DROPPED_ARTICLE_COUNT,droppedCount) ;
+    notice ("ME dropped %ld articles", droppedCount) ;
      
   droppedFileCount = (droppedFileCount + 1) % 26 ;
   droppedCount = 0 ;
@@ -801,7 +769,7 @@ static void dropArticle (const char *peerName, Article article)
 
   if (!logged)
     {
-      syslog (LOG_WARNING,DROPPED_LOCATION,dropArtFile) ;
+      warn ("ME dropping articles into %s", dropArtFile) ;
       logged = true ;
     }
   
@@ -813,5 +781,23 @@ static void dropArticle (const char *peerName, Article article)
 
 static void listenerCleanup (void)
 {
-  FREE (dropArtFile) ;
+  free (dropArtFile) ;
+  dropArtFile = NULL ;
 }
+
+/*
+**  Copyright 1996 by the Internet Software Consortium
+**
+**  Permission to use, copy, modify, and distribute this software for any
+**  purpose with or without fee is hereby granted, provided that the above
+**  copyright notice and this permission notice appear in all copies.
+**
+**  THE SOFTWARE IS PROVIDED "AS IS" AND INTERNET SOFTWARE CONSORTIUM
+**  DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL
+**  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.  IN NO EVENT SHALL
+**  INTERNET SOFTWARE CONSORTIUM BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
+**  OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
+**  USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
+**  OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
+**  PERFORMANCE OF THIS SOFTWARE.
+*/
