@@ -1,4 +1,4 @@
-/*  $Id: daemonize.c 7585 2006-11-21 09:37:51Z eagle $
+/*  $Id: daemonize.c,v 1.4 2002/08/27 08:01:56 alexk Exp $
 **
 **  Become a long-running daemon.
 **
@@ -18,7 +18,7 @@
 #include <sys/stat.h>
 
 #include "inn/messages.h"
-#include "inn/libinn.h"
+#include "libinn.h"
 
 void
 daemonize(const char *path)
@@ -34,21 +34,18 @@ daemonize(const char *path)
     else if (status > 0)
         _exit(0);
 
-    /* setsid() should take care of disassociating from the controlling
-       terminal, and FreeBSD at least doesn't like TIOCNOTTY if you don't
-       already have a controlling terminal.  So only use the older TIOCNOTTY
-       method if setsid() isn't available. */
-#if HAVE_SETSID
-    if (setsid() < 0)
-        syswarn("cant become session leader");
-#elif defined(TIOCNOTTY)
+#ifdef TIOCNOTTY
     fd = open("/dev/tty", O_RDWR);
     if (fd >= 0) {
         if (ioctl(fd, TIOCNOTTY, NULL) < 0)
             syswarn("cant disassociate from the terminal");
         close(fd);
     }
-#endif /* defined(TIOCNOTTY) */
+#endif /* TIOCNOTTY */
+
+#if HAVE_SETSID
+    setsid();
+#endif
 
     if (chdir(path) < 0)
         syswarn("cant chdir to %s", path);
