@@ -1,4 +1,4 @@
-/*  $Id: readin.c 7585 2006-11-21 09:37:51Z eagle $
+/*  $Id: readin.c,v 1.9 2003/01/19 01:15:34 rra Exp $
 **
 */
 
@@ -8,7 +8,7 @@
 #include <fcntl.h>
 #include <sys/stat.h>
 
-#include "inn/libinn.h"
+#include "libinn.h"
 
 
 /*
@@ -19,13 +19,9 @@ int xread(int fd, char *p, off_t i)
 {
     int	                count;
 
-    for ( ; i; p += count, i -= count) {
-        do {
-            count = read(fd, p, i);
-        } while (count == -1 && errno == EINTR);
-        if (count <= 0)
-            return -1;
-    }
+    for ( ; i; p += count, i -= count)
+	if ((count = read(fd, p, i)) <= 0)
+	    return -1;
     return 0;
 }
 
