@@ -1,12 +1,12 @@
-/*  $Revision: 1.2 $
+/*  $Id: xfopena.c,v 1.4 2002/03/31 22:35:47 rra Exp $
 **
 */
-#include <stdio.h>
-#include <sys/types.h>
-#include "configdata.h"
+
+#include "config.h"
 #include "clibrary.h"
 #include <fcntl.h>
 
+#include "libinn.h"
 
 /*
 **  Open a file in append mode.  Since not all fopen's set the O_APPEND
