@@ -1,4 +1,4 @@
-/*  $Id: interface.c,v 1.45.2.7 2002/04/01 08:22:32 rra Exp $
+/*  $Id: interface.c,v 1.70 2003/03/27 23:56:00 rra Exp $
 **
 **  Storage Manager interface
 */
@@ -10,70 +10,74 @@
 #include <time.h>
 
 #include "conffile.h"
+#include "inn/innconf.h"
+#include "inn/wire.h"
 #include "interface.h"
 #include "libinn.h"
-#include "macros.h"
 #include "methods.h"
 #include "paths.h"
 
 typedef enum {INIT_NO, INIT_DONE, INIT_FAIL} INITTYPE;
 typedef struct {
     INITTYPE		initialized;
-    BOOL		configured;
-    BOOL		selfexpire;
-    BOOL		expensivestat;
+    bool		configured;
+    bool		selfexpire;
+    bool		expensivestat;
 } METHOD_DATA;
 
 METHOD_DATA method_data[NUM_STORAGE_METHODS];
 
-STATIC STORAGE_SUB      *subscriptions = NULL;
-STATIC unsigned int     typetoindex[256];
+static STORAGE_SUB      *subscriptions = NULL;
+static unsigned int     typetoindex[256];
 int                     SMerrno;
 char                    *SMerrorstr = NULL;
-STATIC BOOL             ErrorAlloc = FALSE;
-STATIC BOOL             Initialized = FALSE;
-BOOL			SMopenmode = FALSE;
-BOOL			SMpreopen = FALSE;
+static bool             ErrorAlloc = false;
+static bool             Initialized = false;
+bool			SMopenmode = false;
+bool			SMpreopen = false;
 
 /*
 ** Checks to see if the token is valid
 */
-BOOL IsToken(const char *text) {
+bool IsToken(const char *text) {
     const char          *p;
     
     if (!text)
-	return FALSE;
+	return false;
     
     if (strlen(text) != (sizeof(TOKEN) * 2) + 2)
-	return FALSE;
+	return false;
     
     if (text[0] != '@')
-	return FALSE;
+	return false;
 
     if (text[(sizeof(TOKEN) * 2) + 1] != '@')
-	return FALSE;
+	return false;
 
     for (p = text + 1; *p != '@'; p++)
 	if (!isxdigit((int)*p))
-	    return FALSE;
+	    return false;
     
-    return TRUE;
+    return true;
 }
 
 /*
 ** Converts a token to a textual representation for error messages
 ** and the like.
 */
-char *TokenToText(const TOKEN token) {
-    static char         hex[] = "0123456789ABCDEF";
+char *
+TokenToText(const TOKEN token)
+{
+    static const char   hex[] = "0123456789ABCDEF";
     static char         result[(sizeof(TOKEN) * 2) + 3];
-    char                *p;
+    const char          *p;
     char                *q;
-    char                i;
+    size_t              i;
 
     
     result[0] = '@';
-    for (q = result + 1, p = (char *)&token, i = 0; i < sizeof(TOKEN); i++, p++) {
+    for (q = result + 1, p = (const char *) &token, i = 0; i < sizeof(TOKEN);
+         i++, p++) {
 	*q++ = hex[(*p & 0xF0) >> 4];
 	*q++ = hex[*p & 0x0F];
     }
@@ -86,7 +90,7 @@ char *TokenToText(const TOKEN token) {
 /*
 ** Converts a hex digit and converts it to a int
 */
-STATIC int hextodec(const int c) {
+static int hextodec(const int c) {
     return isdigit(c) ? (c - '0') : ((c - 'A') + 10);
 }
 
@@ -117,12 +121,14 @@ TOKEN TextToToken(const char *text) {
 ** containing the article in wire format.  Set *newlen to the length of the
 ** new article.
 */ 
-char *ToWireFmt(const char *article, int len, int *newlen) {
-    int bytes;
+char *
+ToWireFmt(const char *article, size_t len, size_t *newlen)
+{
+    size_t bytes;
     char *newart;
     const char *p;
     char  *dest;
-    BOOL atstartofline=TRUE;
+    bool atstartofline=true;
 
     /* First go thru article and count number of bytes we need. */
     for (bytes = 0, p=article ; p < &article[len] ; ++p) {
@@ -130,26 +136,26 @@ char *ToWireFmt(const char *article, int len, int *newlen) {
 	++bytes;
 	if (*p == '\n') {
 	    ++bytes; /* need another byte for CR */
-	    atstartofline = TRUE; /* next char starts new line */
+	    atstartofline = true; /* next char starts new line */
 	} else {
-	    atstartofline = FALSE;
+	    atstartofline = false;
 	}
     }
     bytes += 3; /* for .\r\n */
-    newart = NEW(char, bytes + 1);
+    newart = xmalloc(bytes + 1);
     *newlen = bytes;
 
     /* now copy the article, making changes */
-    atstartofline = TRUE;
+    atstartofline = true;
     for (p=article, dest=newart ; p < &article[len] ; ++p) {
 	if (*p == '\n') {
 	    *dest++ = '\r';
 	    *dest++ = '\n';
-	    atstartofline = TRUE;
+	    atstartofline = true;
 	} else {
 	    if (atstartofline && *p == '.') *dest++ = '.'; /* add extra . */
 	    *dest++ = *p;
-	    atstartofline = FALSE;
+	    atstartofline = false;
 	}
     }
     *dest++ = '.';
@@ -159,12 +165,14 @@ char *ToWireFmt(const char *article, int len, int *newlen) {
     return newart;
 }
 
-char *FromWireFmt(const char *article, int len, int *newlen) {
-    int bytes;
+char *
+FromWireFmt(const char *article, size_t len, size_t *newlen)
+{
+    size_t bytes;
     char *newart;
     const char *p;
     char *dest;
-    BOOL atstartofline = TRUE;
+    bool atstartofline = true;
 
     /* First go thru article and count number of bytes we need */
     for (bytes = 0, p=article ; p < &article[len] ; ) {
@@ -175,18 +183,18 @@ char *FromWireFmt(const char *article, int len, int *newlen) {
 	if (atstartofline && p < &article[len-1] && *p == '.' && p[1] == '.') {
 	    bytes++; /* only output 1 byte */
 	    p+=2; 
-	    atstartofline = FALSE;
+	    atstartofline = false;
 	} else if (p < &article[len-1] && *p == '\r' && p[1] == '\n') { 
 	    bytes++; /* \r\n counts as only one byte in output */
 	    p += 2;
-	    atstartofline = TRUE;
+	    atstartofline = true;
 	} else {
 	    bytes++;
 	    p++;
-	    atstartofline = FALSE;
+	    atstartofline = false;
 	}
     }
-    newart = NEW(char, bytes + 1);
+    newart = xmalloc(bytes + 1);
     *newlen = bytes;
     for (p = article, dest = newart ; p < &article[len]; ) {
 	/* check for terminating .\r\n and if so break */
@@ -195,14 +203,14 @@ char *FromWireFmt(const char *article, int len, int *newlen) {
 	if (atstartofline && p < &article[len-1] && *p == '.' && p[1] == '.') {
 	    *dest++ = '.';
 	    p += 2;
-	    atstartofline = FALSE;
+	    atstartofline = false;
 	} else if (p < &article[len-1] && *p == '\r' && p[1] == '\n') {
 	    *dest++ = '\n';
 	    p += 2;
-	    atstartofline = TRUE;
+	    atstartofline = true;
 	} else {
 	    *dest++ = *p++;
-	    atstartofline = FALSE;
+	    atstartofline = false;
 	}
     }
     *dest = '\0';
@@ -212,21 +220,25 @@ char *FromWireFmt(const char *article, int len, int *newlen) {
 /*
 **  get Xref header without pathhost
 */
-STATIC char *GetXref(ARTHANDLE *art) {
-  char	*p, *p1;
-  char	*q;
+static char *
+GetXref(ARTHANDLE *art)
+{
+  const char *p, *p1;
+  const char *q;
   char	*buff;
-  BOOL	Nocr;
+  bool	Nocr = false;
 
-  if ((p = q = (char *)HeaderFindMem(art->data, art->len, "xref", sizeof("xref")-1)) == NULL)
+  p = wire_findheader(art->data, art->len, "xref");
+  if (p == NULL)
     return NULL;
+  q = p;
   for (p1 = NULL; p < art->data + art->len; p++) {
     if (p1 != (char *)NULL && *p1 == '\r' && *p == '\n') {
-      Nocr = FALSE;
+      Nocr = false;
       break;
     }
     if (*p == '\n') {
-      Nocr = TRUE;
+      Nocr = true;
       break;
     }
     p1 = p;
@@ -244,7 +256,7 @@ STATIC char *GetXref(ARTHANDLE *art) {
   for (q++; (*q == ' ') && (q < p); q++);
   if (q == p)
     return NULL;
-  buff = NEW(char, p - q + 1);
+  buff = xmalloc(p - q + 1);
   memcpy(buff, q, p - q);
   buff[p - q] = '\0';
   return buff;
@@ -254,7 +266,7 @@ STATIC char *GetXref(ARTHANDLE *art) {
 **  Split newsgroup and returns artnum
 **  or 0 if there are no newsgroup.
 */
-STATIC ARTNUM GetGroups(char *Xref) {
+static ARTNUM GetGroups(char *Xref) {
   char	*p;
 
   if ((p = strchr(Xref, ':')) == NULL)
@@ -263,21 +275,6 @@ STATIC ARTNUM GetGroups(char *Xref) {
   return ((ARTNUM)atoi(p));
 }
 
-/*
-**  Searches through the given string and find the begining of the
-**  message body and returns that if it finds it.  If not, it returns
-**  NULL.
-*/
-char *SMFindBody(char *article, int len) {
-    char                *p;
-
-    for (p = article; p < (article + len - 4); p++) {
-	if (!memcmp(p, "\r\n\r\n", 4))
-	    return p+4;
-    }
-    return NULL;
-}
-
 STORAGE_SUB *SMGetConfig(STORAGETYPE type, STORAGE_SUB *sub) {
     if (sub == (STORAGE_SUB *)NULL)
 	sub = subscriptions;
@@ -336,6 +333,7 @@ static time_t ParseTime(char *tmbuf)
 #define SMclass   13
 #define SMexpire  14
 #define SMoptions 15
+#define SMexactmatch 16
 
 static CONFTOKEN smtoks[] = {
   { SMlbrace,	"{" },
@@ -346,47 +344,55 @@ static CONFTOKEN smtoks[] = {
   { SMclass,	"class:" },
   { SMexpire,	"expires:" },
   { SMoptions,	"options:" },
+  { SMexactmatch,	"exactmatch:" },
   { 0, 0 }
 };
 
 /* Open the config file and parse it, generating the policy data */
-static BOOL SMreadconfig(void) {
+static bool
+SMreadconfig(void)
+{
     CONFFILE            *f;
     CONFTOKEN           *tok;
     int			type;
     int                 i;
     char                *p;
     char                *q;
-    char                *method;
-    char                *patterns = NULL;
-    int                 minsize;
-    int                 maxsize;
-    time_t		minexpire;
-    time_t		maxexpire;
-    int                 class;
+    char                *path;
+    char                *method = NULL;
+    char                *pattern = NULL;
+    size_t              minsize = 0;
+    size_t              maxsize = 0;
+    time_t		minexpire = 0;
+    time_t		maxexpire = 0;
+    int                 class = 0;
     STORAGE_SUB         *sub = NULL;
     STORAGE_SUB         *prev = NULL;
     char		*options = 0;
     int			inbrace;
+    bool		exactmatch = false;
 
     /* if innconf isn't already read in, do so. */
     if (innconf == NULL) {
-	if (ReadInnConf() < 0) {
+        if (!innconf_read(NULL)) {
 	    SMseterror(SMERR_INTERNAL, "ReadInnConf() failed");
-	    return FALSE;
+	    return false;
 	}
     }
 
     for (i = 0; i < NUM_STORAGE_METHODS; i++) {
 	method_data[i].initialized = INIT_NO;
-	method_data[i].configured = FALSE;
+	method_data[i].configured = false;
     }
-    if ((f = CONFfopen(cpcatpath(innconf->pathetc, _PATH_STORAGECTL))) == NULL) {
+    path = concatpath(innconf->pathetc, _PATH_STORAGECTL);
+    f = CONFfopen(path);
+    if (f == NULL) {
 	SMseterror(SMERR_UNDEFINED, NULL);
-	syslog(L_ERROR, "SM Could not open %s: %m",
-			cpcatpath(innconf->pathetc, _PATH_STORAGECTL));
-	return FALSE;
+	syslog(L_ERROR, "SM Could not open %s: %m", path);
+        free(path);
+	return false;
     }
+    free(path);
     
     inbrace = 0;
     while ((tok = CONFgettoken(smtoks, f)) != NULL) {
@@ -394,27 +400,29 @@ static BOOL SMreadconfig(void) {
 	    if (tok->type != SMmethod) {
 		SMseterror(SMERR_CONFIG, "Expected 'method' keyword");
 		syslog(L_ERROR, "SM expected 'method' keyword, line %d", f->lineno);
-		return FALSE;
+		return false;
 	    }
 	    if ((tok = CONFgettoken(0, f)) == NULL) {
 		SMseterror(SMERR_CONFIG, "Expected method name");
 		syslog(L_ERROR, "SM expected method name, line %d", f->lineno);
-		return FALSE;
+		return false;
 	    }
-	    method = COPY(tok->name);
+	    method = xstrdup(tok->name);
 	    if ((tok = CONFgettoken(smtoks, f)) == NULL || tok->type != SMlbrace) {
 		SMseterror(SMERR_CONFIG, "Expected '{'");
 		syslog(L_ERROR, "SM Expected '{', line %d", f->lineno);
-		return FALSE;
+		return false;
 	    }
 	    inbrace = 1;
 	    /* initialize various params to defaults. */
 	    minsize = 0;
 	    maxsize = 0; /* zero means no limit */
 	    class = 0;
-	    options = (char *)NULL;
+            pattern = NULL;
+	    options = NULL;
 	    minexpire = 0;
 	    maxexpire = 0;
+	    exactmatch = false;
 
 	} else {
 	    type = tok->type;
@@ -424,20 +432,20 @@ static BOOL SMreadconfig(void) {
 		if ((tok = CONFgettoken(0, f)) == NULL) {
 		    SMseterror(SMERR_CONFIG, "Keyword with no value");
 		    syslog(L_ERROR, "SM keyword with no value, line %d", f->lineno);
-		    return FALSE;
+		    return false;
 		}
 		p = tok->name;
 		switch(type) {
 		  case SMgroups:
-		    if (patterns)
-			DISPOSE(patterns);
-		    patterns = COPY(tok->name);
+		    if (pattern)
+			free(pattern);
+		    pattern = xstrdup(tok->name);
 		    break;
 		  case SMsize:
-		    minsize = atoi(p);
+		    minsize = strtoul(p, NULL, 10);
 		    if ((p = strchr(p, ',')) != NULL) {
 			p++;
-			maxsize = atoi(p);
+			maxsize = strtoul(p, NULL, 10);
 		    }
 		    break;
 		  case SMclass:
@@ -453,77 +461,63 @@ static BOOL SMreadconfig(void) {
 		    break;
 		  case SMoptions:
 		    if (options)
-			DISPOSE(options);
-		    options = COPY(p);
+			free(options);
+		    options = xstrdup(p);
+		    break;
+		  case SMexactmatch:
+		    if (strcasecmp(p, "true") == 0
+                        || strcasecmp(p, "yes") == 0
+                        || strcasecmp(p, "on") == 0)
+			exactmatch = true;
 		    break;
 		  default:
 		    SMseterror(SMERR_CONFIG, "Unknown keyword in method declaration");
 		    syslog(L_ERROR, "SM Unknown keyword in method declaration, line %d: %s", f->lineno, tok->name);
-		    DISPOSE(method);
-		    return FALSE;
+		    free(method);
+		    return false;
 		    break;
 		}
 	    }
 	}
 	if (!inbrace) {
 	    /* just finished a declaration */
-	    sub = NEW(STORAGE_SUB, 1);
+	    sub = xmalloc(sizeof(STORAGE_SUB));
 	    sub->type = TOKEN_EMPTY;
 	    for (i = 0; i < NUM_STORAGE_METHODS; i++) {
 		if (!strcasecmp(method, storage_methods[i].name)) {
 		    sub->type = storage_methods[i].type;
-		    method_data[i].configured = TRUE;
+		    method_data[i].configured = true;
 		    break;
 		}
 	    }
 	    if (sub->type == TOKEN_EMPTY) {
 		SMseterror(SMERR_CONFIG, "Invalid storage method name");
 		syslog(L_ERROR, "SM no configured storage methods are named '%s'", method);
-		DISPOSE(options);
-		DISPOSE(sub);
-		return FALSE;
+		free(options);
+		free(sub);
+		return false;
 	    }
-	    if (!patterns) {
-		SMseterror(SMERR_CONFIG, "patterns not defined");
-		syslog(L_ERROR, "SM no patterns defined");
-		DISPOSE(options);
-		DISPOSE(sub);
-		return FALSE;
+	    if (!pattern) {
+		SMseterror(SMERR_CONFIG, "pattern not defined");
+		syslog(L_ERROR, "SM no pattern defined");
+		free(options);
+		free(sub);
+		return false;
 	    }
+            sub->pattern = pattern;
 	    sub->minsize = minsize;
 	    sub->maxsize = maxsize;
 	    sub->class = class;
 	    sub->options = options;
 	    sub->minexpire = minexpire;
 	    sub->maxexpire = maxexpire;
+	    sub->exactmatch = exactmatch;
 
-	    DISPOSE(method);
+	    free(method);
 	    method = 0;
-	    
-	
-	    /* Count the number of patterns and allocate space*/
-	    for (i = 1, p = patterns; *p && (p = strchr(p+1, ',')); i++);
 
-	    sub->numpatterns = i;
-	    sub->patterns = NEW(char *, i);
 	    if (!prev)
 		subscriptions = sub;
-
-	    /* Store the patterns. */
-	    for (i = 0, p = strtok(patterns, ","); p != NULL; i++, p = strtok(NULL, ","))
-		sub->patterns[i] = COPY(p);
-	    DISPOSE(patterns);
-	    patterns = NULL;
-
-	    if (i != sub->numpatterns) {
-	      /* Spurious ',' */
-	      SMseterror(SMERR_CONFIG, "extra ',' in pattern");
-	      syslog(L_ERROR, "SM extra ',' in pattern");
-	      DISPOSE(options);
-	      DISPOSE(sub);
-	      return FALSE;
-	    }
-
 	    if (prev)
 		prev->next = sub;
 	    prev = sub;
@@ -531,49 +525,49 @@ static BOOL SMreadconfig(void) {
 	}
     }
     
-    (void)CONFfclose(f);
+    CONFfclose(f);
 
-    return TRUE;
+    return true;
 }
 
 /*
 ** setup storage api environment (open mode etc.)
 */
-BOOL SMsetup(SMSETUP type, void *value) {
+bool SMsetup(SMSETUP type, void *value) {
     if (Initialized)    
-	return FALSE;
+	return false;
     switch (type) {
     case SM_RDWR:
-	SMopenmode = *(BOOL *)value;
+	SMopenmode = *(bool *)value;
 	break;
     case SM_PREOPEN:
-	SMpreopen = *(BOOL *)value;
+	SMpreopen = *(bool *)value;
 	break;
     default:
-	return FALSE;
+	return false;
     }
-    return TRUE;
+    return true;
 }
 
 /*
 ** Calls the setup function for all of the configured methods and returns
-** TRUE if they all initialize ok, FALSE if they don't
+** true if they all initialize ok, false if they don't
 */
-BOOL SMinit(void) {
+bool SMinit(void) {
     int                 i;
-    BOOL		allok = TRUE;
-    static		BOOL once = FALSE;
+    bool		allok = true;
+    static		bool once = false;
     SMATTRIBUTE		smattr;
 
     if (Initialized)
-	return TRUE;
+	return true;
     
-    Initialized = TRUE;
+    Initialized = true;
     
     if (!SMreadconfig()) {
 	SMshutdown();
-	Initialized = FALSE;
-	return FALSE;
+	Initialized = false;
+	return false;
     }
 
     for (i = 0; i < NUM_STORAGE_METHODS; i++) {
@@ -584,174 +578,123 @@ BOOL SMinit(void) {
 		method_data[i].expensivestat = smattr.expensivestat;
 	    } else {
 		method_data[i].initialized = INIT_FAIL;
-		method_data[i].selfexpire = FALSE;
-		method_data[i].expensivestat = TRUE;
+		method_data[i].selfexpire = false;
+		method_data[i].expensivestat = true;
 		syslog(L_ERROR, "SM storage method '%s' failed initialization", storage_methods[i].name);
-		allok = FALSE;
+		allok = false;
 	    }
 	}
 	typetoindex[storage_methods[i].type] = i;
     }
     if (!allok) {
 	SMshutdown();
-	Initialized = FALSE;
+	Initialized = false;
 	SMseterror(SMERR_UNDEFINED, "one or more storage methods failed initialization");
 	syslog(L_ERROR, "SM one or more storage methods failed initialization");
-	return FALSE;
+	return false;
     }
     if (!once && atexit(SMshutdown) < 0) {
 	SMshutdown();
-	Initialized = FALSE;
+	Initialized = false;
 	SMseterror(SMERR_UNDEFINED, NULL);
-	return FALSE;
+	return false;
     }
-    once = TRUE;
-    return TRUE;
+    once = true;
+    return true;
 }
 
-static BOOL InitMethod(STORAGETYPE method) {
+static bool InitMethod(STORAGETYPE method) {
     SMATTRIBUTE		smattr;
 
     if (!Initialized)
 	if (!SMreadconfig()) {
-	    Initialized = FALSE;
-	    return FALSE;
+	    Initialized = false;
+	    return false;
 	}
-    Initialized = TRUE;
+    Initialized = true;
     
     if (method_data[method].initialized == INIT_DONE)
-	return TRUE;
+	return true;
 
     if (method_data[method].initialized == INIT_FAIL)
-	return FALSE;
+	return false;
 
     if (!method_data[method].configured) {
 	method_data[method].initialized = INIT_FAIL;
 	SMseterror(SMERR_UNDEFINED, "storage method is not configured.");
-	return FALSE;
+	return false;
     }
     if (!storage_methods[method].init(&smattr)) {
 	method_data[method].initialized = INIT_FAIL;
-	method_data[method].selfexpire = FALSE;
-	method_data[method].expensivestat = TRUE;
+	method_data[method].selfexpire = false;
+	method_data[method].expensivestat = true;
 	SMseterror(SMERR_UNDEFINED, "Could not initialize storage method late.");
-	return FALSE;
+	return false;
     }
     method_data[method].initialized = INIT_DONE;
     method_data[method].selfexpire = smattr.selfexpire;
     method_data[method].expensivestat = smattr.expensivestat;
-    return TRUE;
+    return true;
 }
 
-static BOOL MatchGroups(const char *g, int num, char **patterns) {
-    char                *group;
-    char                *groups;
-    char		*groupsep, *q;
-    const char          *p;
-    int                 i;
-    BOOL                wanted = FALSE;
-    BOOL                poisoned = FALSE;
+static bool
+MatchGroups(const char *g, int len, const char *pattern, bool exactmatch)
+{
+    char *group, *groups, *q;
+    int i, lastwhite;
+    enum uwildmat matched;
+    bool wanted = false;
 
-    /* Find the end of the line */
-    for (p = g+1; (*p != '\n') && (*(p - 1) != '\r'); p++);
+    q = groups = xmalloc(len + 1);
+    for (lastwhite = -1,  i = 0 ; i < len ; i++) {
+	/* trim white chars */
+	if (g[i] == '\r' || g[i] == '\n' || g[i] == ' ' || g[i] == '\t') {
+	    if (lastwhite + 1 != i)
+	        *q++ = ' ';
+	    lastwhite = i;
+	} else
+	    *q++ = g[i];
+    }
+    *q = '\0';
 
-    groups = NEW(char, p - g);
-    memcpy(groups, g, p - g - 1);
-    groups[p - g - 1] = '\0';
-
-    if (innconf->storeonxref)
-	groupsep = " ";
-    else
-	groupsep = ",";
-    for (group = strtok(groups, groupsep); group != NULL; group = strtok(NULL, groupsep)) {
-	if (innconf->storeonxref && ((q = strchr(group, ':')) != (char *)NULL))
-	    *q = '\0';
-	for (i = 0; i < num; i++) {
-	    switch (patterns[i][0]) {
-	    case '!':
-		if (wildmat(group, &patterns[i][1]))
-                    wanted = FALSE;
-                break;
-	    case '@':
-		if (wildmat(group, &patterns[i][1])) {
-                    wanted = FALSE;
-                    poisoned = TRUE;
-		}
-		break;
-	    default:
-		if (wildmat(group, patterns[i])) {
-		    wanted = TRUE;
-                    poisoned = FALSE;
-                }
-                break;
-	    }
+    group = strtok(groups, " ,");
+    while (group != NULL) {
+        q = strchr(group, ':');
+        if (q != NULL)
+            *q = '\0';
+        matched = uwildmat_poison(group, pattern);
+        if (matched == UWILDMAT_POISON || (exactmatch && !matched)) {
+	    free(groups);
+	    return false;
 	}
-        if (poisoned) {
-            DISPOSE(groups);
-            return FALSE;
-        }
+        if (matched == UWILDMAT_MATCH)
+            wanted = true;
+        group = strtok(NULL, " ,");
     }
 
-    DISPOSE(groups);
+    free(groups);
     return wanted;
 }
 
 STORAGE_SUB *SMgetsub(const ARTHANDLE article) {
     STORAGE_SUB         *sub;
-    char                *groups;
-    char		*expire;
-    time_t		expiretime;
 
-    if (!article.data || !article.len) {
+    if (article.len == 0) {
 	SMseterror(SMERR_BADHANDLE, NULL);
 	return NULL;
     }
 
-    if (innconf->storeonxref) {
-	if ((groups = (char *)HeaderFindMem(article.data, article.len, "Xref", 4)) == NULL) {
-	    errno = 0;
-	    SMseterror(SMERR_UNDEFINED, "Could not find Xref header");
-	    return NULL;
-	}
-	/* skip pathhost */
-	if ((groups = strchr(groups, ' ')) == NULL) {
-	    errno = 0;
-	    SMseterror(SMERR_UNDEFINED, "Could not find pathhost in Xref header");
-	    return NULL;
-	}
-	for (groups++; *groups == ' '; groups++);
-    } else {
-	if ((groups = (char *)HeaderFindMem(article.data, article.len, "Newsgroups", 10)) == NULL) {
-	    errno = 0;
-	    SMseterror(SMERR_UNDEFINED, "Could not find Newsgroups header");
-	    return NULL;
-	}
-    }
-
-    expiretime = 0;
-    if ((expire = (char *)HeaderFindMem(article.data, article.len, "Expires", 7))) {
-	/* optionally parse expire header */
-	char *x, *p;
-	for (p = expire+1; (*p != '\n') && (*(p - 1) != '\r'); p++);
-	x = NEW(char, p - expire);
-    	memcpy(x, expire, p - expire - 1);
-    	x[p - expire - 1] = '\0';
-	
-	expiretime = parsedate(x, NULL);
-	if (expiretime == -1)
-	    expiretime = 0;
-	else
-	    expiretime -= time(0);
-	DISPOSE(x);
-    }
+    if (article.groups == NULL)
+	return NULL;
 
     for (sub = subscriptions; sub != NULL; sub = sub->next) {
 	if (!(method_data[typetoindex[sub->type]].initialized == INIT_FAIL) &&
 	    (article.len >= sub->minsize) &&
 	    (!sub->maxsize || (article.len <= sub->maxsize)) &&
-	    (!sub->minexpire || expiretime >= sub->minexpire) &&
-	    (!sub->maxexpire || (expiretime <= sub->maxexpire)) &&
-	    MatchGroups(groups, sub->numpatterns, sub->patterns)) {
+	    (!sub->minexpire || article.expires >= sub->minexpire) &&
+	    (!sub->maxexpire || (article.expires <= sub->maxexpire)) &&
+	    MatchGroups(article.groups, article.groupslen, sub->pattern,
+                        sub->exactmatch)) {
 	    if (InitMethod(typetoindex[sub->type]))
 		return sub;
 	}
@@ -839,24 +782,24 @@ void SMfreearticle(ARTHANDLE *article) {
     storage_methods[typetoindex[article->type]].freearticle(article);
 }
 
-BOOL SMcancel(TOKEN token) {
+bool SMcancel(TOKEN token) {
     if (!SMopenmode) {
 	SMseterror(SMERR_INTERNAL, "read only storage api");
-	return FALSE;
+	return false;
     }
     if (method_data[typetoindex[token.type]].initialized == INIT_FAIL) {
 	SMseterror(SMERR_UNINIT, NULL);
-	return FALSE;
+	return false;
     }
     if (method_data[typetoindex[token.type]].initialized == INIT_NO && !InitMethod(typetoindex[token.type])) {
 	SMseterror(SMERR_UNINIT, NULL);
 	syslog(L_ERROR, "SM can't cancel article with uninitialized method");
-	return FALSE;
+	return false;
     }
     return storage_methods[typetoindex[token.type]].cancel(token);
 }
 
-BOOL SMprobe(PROBETYPE type, TOKEN *token, void *value) {
+bool SMprobe(PROBETYPE type, TOKEN *token, void *value) {
     struct artngnum	*ann;
     ARTHANDLE		*art;
 
@@ -866,53 +809,53 @@ BOOL SMprobe(PROBETYPE type, TOKEN *token, void *value) {
     case SMARTNGNUM:
 	if (method_data[typetoindex[token->type]].initialized == INIT_FAIL) {
 	    SMseterror(SMERR_UNINIT, NULL);
-	    return FALSE;
+	    return false;
 	}
 	if (method_data[typetoindex[token->type]].initialized == INIT_NO && !InitMethod(typetoindex[token->type])) {
 	    SMseterror(SMERR_UNINIT, NULL);
 	    syslog(L_ERROR, "SM can't cancel article with uninitialized method");
-	    return FALSE;
+	    return false;
 	}
 	if ((ann = (struct artngnum *)value) == NULL)
-	    return FALSE;
+	    return false;
 	ann->groupname = NULL;
 	if (storage_methods[typetoindex[token->type]].ctl(type, token, value)) {
 	    if (ann->artnum != 0) {
 		/* set by storage method */
-		return TRUE;
+		return true;
 	    } else {
 		art = storage_methods[typetoindex[token->type]].retrieve(*token, RETR_HEAD);
 		if (art == NULL) {
 		    if (ann->groupname != NULL)
-			DISPOSE(ann->groupname);
+			free(ann->groupname);
 		    storage_methods[typetoindex[token->type]].freearticle(art);
-		    return FALSE;
+		    return false;
 		}
 		if ((ann->groupname = GetXref(art)) == NULL) {
 		    if (ann->groupname != NULL)
-			DISPOSE(ann->groupname);
+			free(ann->groupname);
 		    storage_methods[typetoindex[token->type]].freearticle(art);
-		    return FALSE;
+		    return false;
 		}
 		storage_methods[typetoindex[token->type]].freearticle(art);
 		if ((ann->artnum = GetGroups(ann->groupname)) == 0) {
 		    if (ann->groupname != NULL)
-			DISPOSE(ann->groupname);
-		    return FALSE;
+			free(ann->groupname);
+		    return false;
 		}
-		return TRUE;
+		return true;
 	    }
 	} else {
-	    return FALSE;
+	    return false;
 	}
     case EXPENSIVESTAT:
 	return (method_data[typetoindex[token->type]].expensivestat);
     default:
-	return FALSE;
+	return false;
     }
 }
 
-BOOL SMflushcacheddata(FLUSHTYPE type) {
+bool SMflushcacheddata(FLUSHTYPE type) {
     int		i;
 
     for (i = 0; i < NUM_STORAGE_METHODS; i++) {
@@ -920,7 +863,7 @@ BOOL SMflushcacheddata(FLUSHTYPE type) {
 	    !storage_methods[i].flushcacheddata(type))
 	    syslog(L_ERROR, "SM can't flush cached data method '%s'", storage_methods[i].name);
     }
-    return TRUE;
+    return true;
 }
 
 void SMprintfiles(FILE *file, TOKEN token, char **xref, int ngroups) {
@@ -946,26 +889,23 @@ void SMshutdown(void) {
 	if (method_data[i].initialized == INIT_DONE) {
 	    storage_methods[i].shutdown();
 	    method_data[i].initialized = INIT_NO;
-	    method_data[i].configured = FALSE;
+	    method_data[i].configured = false;
 	}
     while (subscriptions) {
 	old = subscriptions;
 	subscriptions = subscriptions->next;
-	for (i = 0; i < old->numpatterns; i++) {
-	  DISPOSE(old->patterns[i]);
-	}
-	DISPOSE(old->patterns);
-	DISPOSE(old->options);
-	DISPOSE(old);
+	free(old->pattern);
+	free(old->options);
+	free(old);
     }
-    Initialized = FALSE;
+    Initialized = false;
 }
 
 void SMseterror(int errornum, char *error) {
     if (ErrorAlloc)
-	DISPOSE(SMerrorstr);
+	free(SMerrorstr);
 
-    ErrorAlloc = FALSE;
+    ErrorAlloc = false;
     
     if ((errornum == SMERR_UNDEFINED) && (errno == ENOENT))
 	errornum = SMERR_NOENT;
@@ -975,8 +915,8 @@ void SMseterror(int errornum, char *error) {
     if (error == NULL) {
 	switch (SMerrno) {
 	case SMERR_UNDEFINED:
-	    SMerrorstr = COPY(strerror(errno));
-	    ErrorAlloc = TRUE;
+	    SMerrorstr = xstrdup(strerror(errno));
+	    ErrorAlloc = true;
 	    break;
 	case SMERR_INTERNAL:
 	    SMerrorstr = "Internal error";
@@ -1009,8 +949,8 @@ void SMseterror(int errornum, char *error) {
 	    SMerrorstr = "Undefined error";
 	}
     } else {
-	SMerrorstr = COPY(error);
-	ErrorAlloc = TRUE;
+	SMerrorstr = xstrdup(error);
+	ErrorAlloc = true;
     }
 }
 
