@@ -1,4 +1,4 @@
-/*  $Id: tdx-data.c 9659 2014-08-30 08:08:11Z iulius $
+/*  $Id: tdx-data.c,v 1.27 2003/04/09 01:40:40 rra Exp $
 **
 **  Overview data file handling for the tradindexed overview method.
 **
@@ -22,15 +22,14 @@
 #include <fcntl.h>
 #include <sys/stat.h>
 
-#include "inn/fdflag.h"
 #include "inn/history.h"
 #include "inn/innconf.h"
 #include "inn/messages.h"
 #include "inn/mmap.h"
-#include "inn/libinn.h"
-#include "inn/ov.h"
+#include "libinn.h"
+#include "ov.h"
 #include "ovinterface.h"
-#include "inn/storage.h"
+#include "storage.h"
 #include "tdx-private.h"
 #include "tdx-structure.h"
 
@@ -68,52 +67,24 @@ group_path(const char *group)
     size_t length;
     const char *gp;
 
-    /* The path of the data files for news.groups is dir/n/g/news.groups.  In
-       other words, the first letter of each component becomes a directory.
-       The length of the path is therefore the length of the base overview
-       directory path, one character for the slash, two characters for the
-       first letter and initial slash, two characters for each hierarchical
-       level of the group, and then the length of the group name.
-
-       For robustness, we want to handle leading or multiple consecutive
-       periods.  We only recognize a new hierarchical level after a string of
-       periods (which doesn't end the group name). */
     length = strlen(innconf->pathoverview);
     for (gp = group; *gp != '\0'; gp++)
-        if (*gp == '.') {
-            if (gp[1] == '.' || gp[0] == '\0')
-                continue;
+        if (*gp == '.')
             length += 2;
-        }
-    length += 1 + 2 + strlen(group) + 1;
+    length += 2 + 1 + strlen(group) + 1;
     path = xmalloc(length);
     strlcpy(path, innconf->pathoverview, length);
     p = path + strlen(innconf->pathoverview);
-
-    /* Generate the hierarchical directories. */
-    if (*group != '.' && *group != '\0') {
-        *p++ = '/';
-        *p++ = *group;
-    }
-    for (gp = strchr(group, '.'); gp != NULL; gp = strchr(gp, '.')) {
-        gp++;
-        if (gp == group + 1)
-            continue;
-        if (*gp != '\0' && *gp != '.' && *gp != '/') {
+    for (gp = group; gp != NULL; gp = strchr(gp, '.')) {
+        if (gp != group)
+            gp++;
+        if (*gp != '\0') {
             *p++ = '/';
             *p++ = *gp;
         }
     }
     *p++ = '/';
-
-    /* Finally, append the group name to the generated path and then replace
-       all slashes with commas.  Commas have the advantage of being clearly
-       illegal in newsgroup names because of the syntax of the Newsgroups
-       header, but aren't shell metacharacters. */
     strlcpy(p, group, length - (p - path));
-    for (; *p != '\0'; p++)
-        if (*p == '/')
-            *p = ',';
     return path;
 }
 
@@ -180,7 +151,7 @@ file_open_index(struct group_data *data, const char *suffix)
         return false;
     }
     data->indexinode = st.st_ino;
-    fdflag_close_exec(data->indexfd, true);
+    close_on_exec(data->indexfd, true);
     return true;
 }
 
@@ -199,7 +170,7 @@ file_open_data(struct group_data *data, const char *suffix)
     data->datafd = file_open(data->path, suffix, data->writable, true);
     if (data->datafd < 0)
         return false;
-    fdflag_close_exec(data->datafd, true);
+    close_on_exec(data->datafd, true);
     return true;
 }
 
@@ -216,7 +187,6 @@ tdx_data_new(const char *group, bool writable)
     data = xmalloc(sizeof(struct group_data));
     data->path = group_path(group);
     data->writable = writable;
-    data->remapoutoforder = false;
     data->high = 0;
     data->base = 0;
     data->indexfd = -1;
@@ -426,8 +396,6 @@ tdx_article_entry(struct group_data *data, ARTNUM article, ARTNUM high)
         unmap_index(data);
         map_index(data);
         data->high = high;
-        /* Mark the data file to be remapped for the next opensearch. */
-        data->remapoutoforder = true;
     } else if (innconf->nfsreader && stale_index(data))
         unmap_index(data);
     if (data->index == NULL)
@@ -462,14 +430,11 @@ tdx_search_open(struct group_data *data, ARTNUM start, ARTNUM end, ARTNUM high)
     if (end < start)
         return NULL;
 
-    if ((end > data->high && high > data->high) || data->remapoutoforder) {
-        data->remapoutoforder = false;
-        unmap_data(data);
+    if (end > data->high && high > data->high) {
         unmap_index(data);
         map_index(data);
         data->high = high;
     }
-
     if (start > data->high)
         return NULL;
 
@@ -519,19 +484,19 @@ tdx_search(struct search *search, struct article *artdata)
     if (search->current > search->limit || search->current > max)
         return false;
 
-    /* There is a small chance that remapping the data file could make this
-       offset accessible, but changing the memory location in the middle of
-       a search conflicts with what we return for OVSTATICSEARCH.  And it
-       seems not to be an issue in limited testing, although write caching
-       that leads to on-disk IDX and DAT being out of sync could trigger a
-       problem here. */
+    /* Make sure that the offset into the data file is sensible, and try
+       remapping the data file if the portion the offset is pointing to isn't
+       currently mapped.  Otherwise, warn about possible corruption and return
+       a miss. */
     if (entry->offset + entry->length > search->data->datalen) {
-        search->data->remapoutoforder = true;
-        warn("Invalid or inaccessible entry for article %lu in %s.IDX:"
-             " offset %lu length %lu datalength %lu",
+        unmap_data(search->data);
+        if (!map_data(search->data))
+            return false;
+    }
+    if (entry->offset + entry->length > search->data->datalen) {
+        warn("Invalid entry for article %lu in %s.IDX: offset %lu length %lu",
              search->current + search->data->base, search->data->path,
-             (unsigned long) entry->offset, (unsigned long) entry->length,
-             (unsigned long) search->data->datalen);
+             (unsigned long) entry->offset, (unsigned long) entry->length);
         return false;
     }
 
@@ -568,7 +533,7 @@ tdx_search_close(struct search *search)
 **  constantly repack if the article numbers are clustered around a particular
 **  value but don't come in order.
 */
-static ARTNUM
+ARTNUM
 index_base(ARTNUM artnum)
 {
     return (artnum > 128) ? (artnum - 128) : 1;
@@ -628,31 +593,6 @@ tdx_data_store(struct group_data *data, const struct article *article)
 }
 
 
-/*
-**  Cancels a particular article by removing its index entry.  The data will
-**  still be present in the data file until the next expire run, but it won't
-**  be returned by searches.
-*/
-bool
-tdx_data_cancel(struct group_data *data, ARTNUM artnum)
-{
-    static const struct index_entry empty;
-    off_t offset;
-
-    if (!data->writable)
-        return false;
-    if (data->base == 0 || artnum < data->base || artnum > data->high)
-        return false;
-    offset = (artnum - data->base) * sizeof(struct index_entry);
-    if (xpwrite(data->indexfd, &empty, sizeof(empty), offset) < 0) {
-        syswarn("tradindexed: cannot cancel index record for %lu in %s.IDX",
-                artnum, data->path);
-        return false;
-    }
-    return true;
-}
-
-
 /*
 **  Start the process of packing a group (rewriting its index file so that it
 **  uses a different article base).  Takes the article number of an article
@@ -1115,7 +1055,7 @@ tdx_data_audit(const char *group, struct group_entry *index, bool fix)
     /* All done.  Close things down and flush the data we changed, if
        necessary. */
     if (changed)
-        inn_msync_page(index, sizeof(*index), MS_ASYNC);
+        mapcntl(index, sizeof(*index), MS_ASYNC);
 
  end:
     tdx_data_close(data);
