@@ -1,4 +1,4 @@
-/*  $Id: tdx-group.c 8476 2009-05-18 03:59:10Z eagle $
+/*  $Id: tdx-group.c,v 1.22 2003/01/21 09:19:33 alexk Exp $
 **
 **  Group index handling for the tradindexed overview method.
 **
@@ -95,9 +95,8 @@
 #include "inn/messages.h"
 #include "inn/mmap.h"
 #include "inn/qio.h"
-#include "inn/vector.h"
-#include "inn/libinn.h"
-#include "inn/paths.h"
+#include "libinn.h"
+#include "paths.h"
 #include "tdx-private.h"
 #include "tdx-structure.h"
 
@@ -380,7 +379,7 @@ index_expand(struct group_index *index)
         index->header->freelist.recno = i;
     }
 
-    inn_msync_page(index->header, index_file_size(index->count), MS_ASYNC);
+    mapcntl(index->header, index_file_size(index->count), MS_ASYNC);
     return true;
 }
 
@@ -459,7 +458,7 @@ entry_splice(struct group_entry *entry, int *parent)
 {
     *parent = entry->next.recno;
     entry->next.recno = -1;
-    inn_msync_page(parent, sizeof(*parent), MS_ASYNC);
+    mapcntl(parent, sizeof(*parent), MS_ASYNC);
 }
 
 
@@ -480,8 +479,8 @@ index_add(struct group_index *index, struct group_entry *entry)
     }
     entry->next.recno = index->header->hash[bucket].recno;
     index->header->hash[bucket].recno = entry_loc(index, entry);
-    inn_msync_page(&index->header->hash[bucket], sizeof(struct loc), MS_ASYNC);
-    inn_msync_page(entry, sizeof(*entry), MS_ASYNC);
+    mapcntl(&index->header->hash[bucket], sizeof(struct loc), MS_ASYNC);
+    mapcntl(entry, sizeof(*entry), MS_ASYNC);
 }
 
 
@@ -502,7 +501,7 @@ index_find(struct group_index *index, const char *group)
 	return -1;
     loc = index->header->hash[index_bucket(hash)].recno;
 
-    while (loc >= 0 && loc < index->count) {
+    while (loc >= 0) {
         struct group_entry *entry;
 
         if (loc > index->count && !index_maybe_remap(index, loc))
@@ -529,8 +528,8 @@ freelist_add(struct group_index *index, struct group_entry *entry)
 {
     entry->next.recno = index->header->freelist.recno;
     index->header->freelist.recno = entry_loc(index, entry);
-    inn_msync_page(&index->header->freelist, sizeof(struct loc), MS_ASYNC);
-    inn_msync_page(entry, sizeof(*entry), MS_ASYNC);
+    mapcntl(&index->header->freelist, sizeof(struct loc), MS_ASYNC);
+    mapcntl(entry, sizeof(*entry), MS_ASYNC);
 }
 
 
@@ -549,7 +548,7 @@ index_unlink_hash(struct group_index *index, HASH hash)
     parent = &index->header->hash[index_bucket(hash)].recno;
     current = *parent;
 
-    while (current >= 0 && current < index->count) {
+    while (current >= 0) {
         struct group_entry *entry;
 
         if (current > index->count && !index_maybe_remap(index, current))
@@ -600,7 +599,7 @@ tdx_index_entry(struct group_index *index, const char *group)
         return NULL;
     entry = index->entries + loc;
     if (innconf->tradindexedmmap && innconf->nfsreader)
-	inn_msync_page(entry, sizeof *entry, MS_INVALIDATE);
+	mapcntl(entry, sizeof *entry, MS_INVALIDATE);
     return entry;
 }
 
@@ -630,7 +629,7 @@ tdx_index_add(struct group_index *index, const char *group, ARTNUM low,
         entry = &index->entries[loc];
         if (entry->flag != *flag) {
             entry->flag = *flag;
-            inn_msync_page(entry, sizeof(*entry), MS_ASYNC);
+            mapcntl(entry, sizeof(*entry), MS_ASYNC);
         }
         return true;
     }
@@ -645,7 +644,7 @@ tdx_index_add(struct group_index *index, const char *group, ARTNUM low,
         }
     loc = index->header->freelist.recno;
     index->header->freelist.recno = index->entries[loc].next.recno;
-    inn_msync_page(&index->header->freelist, sizeof(struct loc), MS_ASYNC);
+    mapcntl(&index->header->freelist, sizeof(struct loc), MS_ASYNC);
 
     /* Initialize the entry. */
     entry = &index->entries[loc];
@@ -820,11 +819,11 @@ tdx_data_add(struct group_index *index, struct group_entry *entry,
         old_base = entry->base;
         entry->indexinode = data->indexinode;
         entry->base = data->base;
-        inn_msync_page(entry, sizeof(*entry), MS_ASYNC);
+        mapcntl(entry, sizeof(*entry), MS_ASYNC);
         if (!tdx_data_pack_finish(data)) {
             entry->base = old_base;
             entry->indexinode = old_inode;
-            inn_msync_page(entry, sizeof(*entry), MS_ASYNC);
+            mapcntl(entry, sizeof(*entry), MS_ASYNC);
             goto fail;
         }
     }
@@ -839,14 +838,7 @@ tdx_data_add(struct group_index *index, struct group_entry *entry,
     if (entry->high < article->number)
         entry->high = article->number;
     entry->count++;
-
-    /* Used to know that we have to remap the data file owing to our
-       OVSTATICSEARCH (an article whose number is lower than the highest has
-       been added at the end of the file). */
-    if (data->high > article->number)
-        data->remapoutoforder = true;
-
-    inn_msync_page(entry, sizeof(*entry), MS_ASYNC);
+    mapcntl(entry, sizeof(*entry), MS_ASYNC);
     index_lock_group(index->fd, offset, INN_LOCK_UNLOCK);
     return true;
 
@@ -887,7 +879,7 @@ tdx_index_rebuild_finish(struct group_index *index, struct group_entry *entry,
     *entry = *new;
     entry->indexinode = new_inode;
     new->indexinode = new_inode;
-    inn_msync_page(entry, sizeof(*entry), MS_ASYNC);
+    mapcntl(entry, sizeof(*entry), MS_ASYNC);
     offset = entry - index->entries;
     index_lock_group(index->fd, offset, INN_LOCK_UNLOCK);
     return true;
@@ -937,12 +929,12 @@ tdx_expire(const char *group, ARTNUM *low, struct history *history)
     old_base = entry->base;
     entry->indexinode = new_entry.indexinode;
     entry->base = new_entry.base;
-    inn_msync_page(entry, sizeof(*entry), MS_ASYNC);
+    mapcntl(entry, sizeof(*entry), MS_ASYNC);
     tdx_data_close(data);
     if (!tdx_data_rebuild_finish(group)) {
         entry->base = old_base;
         entry->indexinode = old_inode;
-        inn_msync_page(entry, sizeof(*entry), MS_ASYNC);
+        mapcntl(entry, sizeof(*entry), MS_ASYNC);
         goto fail;
     }
 
@@ -980,15 +972,6 @@ tdx_expire(const char *group, ARTNUM *low, struct history *history)
 struct hashmap {
     HASH hash;
     char *name;
-    char flag;
-};
-
-/* Holds information needed by hash traversal functions.  Right now, this is
-   just the pointer to the group index and a flag saying whether to fix
-   problems or not. */
-struct audit_data {
-    struct group_index *index;
-    bool fix;
 };
 
 
@@ -1043,14 +1026,13 @@ hashmap_load(void)
 {
     struct hash *hash;
     QIOSTATE *active;
-    char *activepath, *line;
-    struct cvector *data = NULL;
+    char *activepath, *line, *p;
     struct stat st;
     size_t hash_size;
     struct hashmap *group;
     HASH grouphash;
 
-    activepath = concatpath(innconf->pathdb, INN_PATH_ACTIVE);
+    activepath = concatpath(innconf->pathdb, _PATH_ACTIVE);
     active = QIOopen(activepath);
     free(activepath);
     if (active == NULL)
@@ -1064,21 +1046,16 @@ hashmap_load(void)
 
     line = QIOread(active);
     while (line != NULL) {
-        data = cvector_split_space(line, data);
-        if (data->count != 4) {
-            warn("tradindexed: malformed active file line %s", line);
-            continue;
-        }
+        p = strchr(line, ' ');
+        if (p != NULL)
+            *p = '\0';
         group = xmalloc(sizeof(struct hashmap));
-        group->name = xstrdup(data->strings[0]);
-        group->flag = data->strings[3][0];
+        group->name = xstrdup(line);
         grouphash = Hash(group->name, strlen(group->name));
         memcpy(&group->hash, &grouphash, sizeof(HASH));
         hash_insert(hash, &group->hash, group);
         line = QIOread(active);
     }
-    if (data != NULL)
-        cvector_free(data);
     QIOclose(active);
     return hash;
 }
@@ -1098,8 +1075,7 @@ tdx_index_print(const char *name, const struct group_entry *entry,
 {
     fprintf(output, "%s %lu %lu %lu %lu %c %lu %lu\n", name, entry->high,
             entry->low, entry->base, (unsigned long) entry->count,
-            entry->flag, (unsigned long) entry->deleted,
-            (unsigned long) entry->indexinode);
+            entry->flag, entry->deleted, (unsigned long) entry->indexinode);
 }
 
 
@@ -1177,7 +1153,7 @@ index_audit_loc(struct group_index *index, int *loc, long number,
 
     if (fix && error) {
         *loc = -1;
-        inn_msync_page(loc, sizeof(*loc), MS_ASYNC);
+        mapcntl(loc, sizeof(*loc), MS_ASYNC);
     }
 }
 
@@ -1195,7 +1171,7 @@ index_audit_deleted(struct group_entry *entry, long number, bool fix)
              number);
         if (fix) {
             HashClear(&entry->hash);
-            inn_msync_page(entry, sizeof(*entry), MS_ASYNC);
+            mapcntl(entry, sizeof(*entry), MS_ASYNC);
         }
     }
 }
@@ -1226,7 +1202,7 @@ index_audit_header(struct group_index *index, bool fix)
         parent = &index->header->hash[bucket].recno;
         index_audit_loc(index, parent, bucket, NULL, fix);
         current = *parent;
-        while (current >= 0 && current < index->count) {
+        while (current != -1) {
             entry = &index->entries[current];
             next = &entry->next.recno;
             if (entry->deleted == 0 && bucket != index_bucket(entry->hash)) {
@@ -1254,8 +1230,6 @@ index_audit_header(struct group_index *index, bool fix)
                     reachable[current] = false;
                 }
             }
-            if (*next == current)
-                break;
             parent = next;
             current = *parent;
         }
@@ -1266,7 +1240,7 @@ index_audit_header(struct group_index *index, bool fix)
     index_audit_loc(index, &index->header->freelist.recno, 0, NULL, fix);
     parent = &index->header->freelist.recno;
     current = *parent;
-    while (current >= 0 && current < index->count) {
+    while (current != -1) {
         entry = &index->entries[current];
         index_audit_deleted(entry, current, fix);
         reachable[current] = true;
@@ -1278,8 +1252,6 @@ index_audit_header(struct group_index *index, bool fix)
             }
         }
         index_audit_loc(index, &entry->next.recno, current, entry, fix);
-        if (entry->next.recno == current)
-            break;
         parent = &entry->next.recno;
         current = *parent;
     }
@@ -1293,11 +1265,8 @@ index_audit_header(struct group_index *index, bool fix)
                 entry = &index->entries[current];
                 if (!HashEmpty(entry->hash) && entry->deleted == 0)
                     index_add(index, entry);
-                else {
-                    HashClear(&entry->hash);
-                    entry->deleted = 0;
+                else
                     freelist_add(index, entry);
-                }
             }
         }
 
@@ -1333,40 +1302,12 @@ index_audit_group(struct group_index *index, struct group_entry *entry,
             freelist_add(index, entry);
         }
     } else {
-        if (entry->flag != group->flag) {
-            entry->flag = group->flag;
-            inn_msync_page(entry, sizeof(*entry), MS_ASYNC);
-        }
         tdx_data_audit(group->name, entry, fix);
     }
     index_lock_group(index->fd, offset, INN_LOCK_UNLOCK);
 }
 
 
-/*
-**  Check to be sure that a given group exists in the overview index, and if
-**  missing, adds it.  Assumes that the index isn't locked, since it calls the
-**  normal functions for adding new groups (this should only be called after
-**  the index has already been repaired, for the same reason).  Called as a
-**  hash traversal function, walking the hash table of groups from the active
-**  file.
-*/
-static void
-index_audit_active(void *value, void *cookie)
-{
-    struct hashmap *group = value;
-    struct audit_data *data = cookie;
-    struct group_entry *entry;
-
-    entry = tdx_index_entry(data->index, group->name);
-    if (entry == NULL) {
-        warn("tradindexed: group %s missing from overview", group->name);
-        if (data->fix)
-            tdx_index_add(data->index, group->name, 0, 0, &group->flag);
-    }
-}
-
-
 /*
 **  Audit the group index for any inconsistencies.  If the argument is true,
 **  also attempt to fix those inconsistencies.
@@ -1381,7 +1322,6 @@ tdx_index_audit(bool fix)
     struct hash *hashmap;
     long bucket;
     struct group_entry *entry;
-    struct audit_data data;
 
     index = tdx_index_open(true);
     if (index == NULL)
@@ -1412,24 +1352,18 @@ tdx_index_audit(bool fix)
 
     /* Okay everything is now mapped and happy.  Validate the header. */
     index_audit_header(index, fix);
-    index_lock(index->fd, INN_LOCK_UNLOCK);
 
     /* Walk all the group entries and check them individually.  To do this, we
        need to map hashes to group names, so load a hash of the active file to
        do that resolution. */
     hashmap = hashmap_load();
-    if (hashmap == NULL) {
-        warn("tradindexed: cannot hash active file");
-        return;
-    }
-    data.index = index;
-    data.fix = fix;
-    hash_traverse(hashmap, index_audit_active, &data);
     for (bucket = 0; bucket < index->count; bucket++) {
         entry = &index->entries[bucket];
         if (HashEmpty(entry->hash) || entry->deleted != 0)
             continue;
         index_audit_group(index, entry, hashmap, fix);
     }
-    hash_free(hashmap);
+    index_lock(index->fd, INN_LOCK_UNLOCK);
+    if (hashmap != NULL)
+        hash_free(hashmap);
 }
